@@ -1,12 +1,12 @@
 
-::Version::setSubversionId {$Id: config.tcl 10049 2008-05-28 22:28:36Z baaazen $}
-
+::Version::setSubversionId {$Id: config.tcl 12033 2010-05-13 20:28:05Z kakaroto $}
 
+package require des
 
 namespace eval ::config {
 
 	proc configDefaults {} {
-		global password auto_path advanced_options custom_emotions
+		global password auto_path advanced_options custom_emotions osspecific_keys locale_codes
 
 		::config::setKey nossl 0			;#Disable the use of SSL, so it doesn't requite TLS package: 0|1
 
@@ -31,6 +31,7 @@ namespace eval ::config {
 		::config::setKey proxy ""			;# If using proxy, proxy host
 		::config::setKey proxytype "http"		;# Proxy type: http|ssl|socks5
 		::config::setKey proxyauthenticate 0		;# SOCKS5 use username/password
+		::config::setKey proxyauthtype "basic"		;# Proxy authentication type (if not NTLM)
 		::config::setKey proxyuser ""		;# user and password for SOCKS5 proxy
 		::config::setKey proxypass ""		;#
 
@@ -56,21 +57,16 @@ namespace eval ::config {
 
 		::config::setKey playbackspeed 100
 
-		#Dir for received files
-		if { [OnDarwin] } {
-		        ::config::setKey receiveddir "[file join $::env(HOME) Desktop]"
-		} elseif { [OnUnix] } {
-			::config::setKey receiveddir "[file join $::env(HOME) amsn_received]"
-		} elseif { [OnWin] } {
-			if {[info exists env(USERPROFILE)]} {
-				::config::setKey receiveddir "[file join $::env(USERPROFILE) amsn_received]"
-			} else {
-				::config::setKey receiveddir "[file join [pwd] amsn_received]"
-			}
+		if {[OnMaemo]} {
+			::config::setKey cam_in_cw 1
 		} else {
-			::config::setKey receiveddir "[file join [pwd] amsn_received]"
+			::config::setKey cam_in_cw 0
 		}
 
+		::config::setKey protocoloverride 0
+
+		::config::setKey receiveddir [::config::getDefaultDownloadDir]
+
 		#Some Autodetected options
 		if { [OnDarwin] } {
 			::config::setKey soundcommand "./utils/macosx/sndplay \$sound";#Soundplayer for Mac OS 10.3-10.4	
@@ -80,14 +76,28 @@ namespace eval ::config {
 			::config::setKey filemanager "open \$location"
 			::config::setKey openfilecommand "open \$file"
 			::config::setKey usesnack 0
+
+			::config::setKey os "mac"
+		} elseif { [OnMaemo] } {
+			::config::setKey soundcommand "aplay \$sound"
+			::config::setKey browser "dbus-send --system --type=method_call --dest=com.nokia.osso_browser /com/nokia/osso_browser com.nokia.osso_browser.open_new_window \"string:\$url\""
+			::config::setKey notifyXoffset 0
+			::config::setKey notifyYoffset 100
+			::config::setKey filemanager "dbus-send --system --type=method_call --dest=com.nokia.osso_filemanager /com/nokia/osso_filemanager com.nokia.osso_filemanager.open_folder \"string:\$location\""
+			::config::setKey openfilecommand "dbus-send --system --type=method_call --dest=com.nokia.osso_filemanager /com/nokia/osso_filemanager com.nokia.osso_filemanager.open_folder \"string:\$file\""
+			::config::setKey usesnack 0
+
+			::config::setKey os "unix"
 		} elseif { [OnUnix] } {
 			::config::setKey soundcommand "play \$sound"
-			::config::setKey browser "mozilla \$url"
+			::config::setKey browser "xdg-open \$url"
 			::config::setKey notifyXoffset 0
 			::config::setKey notifyYoffset 0
-			::config::setKey filemanager "my_filemanager open \$location"
-			::config::setKey openfilecommand ""
+			::config::setKey filemanager "xdg-open \$location"
+			::config::setKey openfilecommand "xdg-open \$file"
 			::config::setKey usesnack 0
+
+			::config::setKey os "unix"
 		} elseif { [OnWin] } {
 			::config::setKey soundcommand "utils/windows/plwav.exe \$sound"
 			::config::setKey browser "explorer \$url"
@@ -97,6 +107,8 @@ namespace eval ::config {
 			::config::setKey openfilecommand "start \$file"
 			::config::setKey usesnack 1
 			#::config::setKey dock 4				;#Set docking to type 4 (windows)
+			
+			::config::setKey os "win"
 		} else {
 			::config::setKey soundcommand ""			;#Sound player command
 			::config::setKey browser ""			;#Browser command
@@ -105,6 +117,8 @@ namespace eval ::config {
 			::config::setKey notifyXoffset 0			;#Notify window offsets
 			::config::setKey notifyYoffset 0
 			::config::setKey usesnack 0			;#Use the Snack library for sounds
+			
+			::config::setKey os "other"
 		}
 
 		::config::setKey autoidle 1				;#Enable/disable auto-idle feature: 0|1
@@ -114,8 +128,9 @@ namespace eval ::config {
 
 		::config::setKey orderusersincreasing 1
 		::config::setKey orderusersbystatus 1
+		::config::setKey orderusersbylogsize 0
 
-		::config::setKey orderbygroup 0			;#Order contacts by group: 0=No | 1=Groups | 2=Hybrid
+		::config::setKey orderbygroup 2			;#Order contacts by group: 0=No | 1=Groups | 2=Hybrid
 		::config::setKey ordergroupsbynormal 1		;#Order groups normal or inverted
 
 		::config::setKey dateformat MDY			;#Change date format (eg Month/Day/Year)
@@ -139,6 +154,7 @@ namespace eval ::config {
 			::config::setKey wingeometry 350x600-0+0			;#Main window geometry
 			::config::setKey backgroundcolor  #D8D8E0		;#AMSN background color
 		}
+                ::config::setKey blinktray 0                                            ;#Make the trayicon blink if there are unread messages
 
 		::config::setKey closingdocks 0						;#Close button minimizes (or docks) main window
 
@@ -204,9 +220,9 @@ namespace eval ::config {
 		::config::setKey logsbydate 1
 		::config::setKey p4c_name ""
 		::config::setKey tabbedchat 2
-		::config::setKey ContainerCloseAction 0
+		::config::setKey closeChatWindowWithTabs 0
 		::config::setKey showMobileGroup 1
-		::config::setKey chatWindowsCloseAfterLogout 0
+		::config::setKey closeChatWindowsAfterLogout 0
 
 		::config::setKey noftpreview 0
 
@@ -229,13 +245,19 @@ namespace eval ::config {
 		::config::setKey rightdelimiter \]		;#Right Timestamps' delimiter ']' by default
 		::config::setKey default_ns_server "messenger.hotmail.com:1863"
 		::config::setKey start_ns_server "messenger.hotmail.com:1863"
+		::config::setKey default_gateway_server "gateway.messenger.hotmail.com"
+		::config::setKey start_gateway_server "gateway.messenger.hotmail.com"		
 		::config::setKey activeautoupdate 1		;#Active the auto update
 		::config::setKey allowbadwords 1		;#Disable censure on nicks
 		::config::setKey enablebanner 1		;#Show or Hide AMSN Banner (By default Show)
 		::config::setKey startontray 0		;#Start amsn on tray icon only (hide contact list)
 		::config::setKey storename 1			;#Store original nick in a variable when go to custom states to revert it when go back
 		::config::setKey strictfonts 0		;#Use strict fonts' size in _ALL_ AMSN's fonts (Disabled by default)
-		::config::setKey sngdblclick 0		;#Use single or double click to open a message window (0 double, 1 single)
+		if {[OnMaemo] } {
+			::config::setKey sngdblclick 1		;#Use single or double click to open a message window (0 double, 1 single)
+		} else {
+			::config::setKey sngdblclick 0		;#Use single or double click to open a message window (0 double, 1 single)
+		}
 		::config::setKey nogap 0			;#Remove the empty line between groups
 		::config::setKey removeempty 0		;#Remove empty groups from the contact list
 		::config::setKey tabtitlenick 1		;#Whether nick or mail is displayed in the tab
@@ -245,10 +267,12 @@ namespace eval ::config {
 		::config::setKey charscounter	1	;#Display typed characters counter
 		::config::setKey checkemail	1	;#Show inbox email notification line
 		::config::setKey recentmsg 0		;#Recent message window closing protection
+		::config::setKey getprofileoffline 0    ;#Use GetProfile for offline users
 		::config::setKey displayp4context 1	;#Accept P4-Context fieds
 		::config::setKey p4contextprefix "" ; #Prefix for P4-Context messages
 		::config::setKey notifytimeout 8000 ; #Number of milisecs before the notify will go away
 		::config::setKey globalnick ""		;#The global custom nickname (pattern), disabled by default
+		::config::setKey contentroaming 1
 		
 		#The place where must be shown the PSM (0: not shown, 1: At the end, 2: In a new line)
 		if {[OnMac]} {
@@ -293,8 +317,38 @@ namespace eval ::config {
 		::config::setKey dynamic_rate 0 				;# Use a dynamic framerate for webcam depending on the timestamps in the ML20 header
 
                 ::config::setKey no_oim_confirmation 0                          ;# Ask or not confirmation to send/read oim messages
+		::config::setKey sound_on_first_message 0			;# Play sound only on the first message received in a chat window
+		::config::setKey colored_text_in_cw 0				;# Show colored and styled text in chatwindows, the value can be changed by ColoredNicks Plugin
 
 
+		::config::setKey epname "aMSN"					;# Endpoint Name.. MSNP18+ name of your current location
+		::config::setKey shownonim 1					;# Show non IM contacts in the contact list
+		::config::setKey groupnonim 0					;# Group non IM contacts in a separate group in the CL
+		::config::setKey showspaces 0					;# Enable/disable showing of the spaces star in the CL
+		::config::setKey showOfflineGroup 1
+		::config::setKey no_blocked_notif 0				;# No notify windows when a user is blocked
+		::config::setKey show_detailed_view 0					;#allow to show detailed contact view in cl
+
+		::config::setKey ABPreferredHost "byrdr.omega.contacts.msn.com"
+		
+		::config::setKey hide_users_in_cw 1	;#in a multichat don't show users in the topcw
+		::config::setKey show_not_in_list 0     ;# Show the 'not in list' icon/emblem
+
+		# Farsight config
+                ::config::setKey fsaudiosrc ""
+                ::config::setKey fsaudiosrcdev ""
+                ::config::setKey fsaudiosink ""
+                ::config::setKey fsaudiosinkdev ""
+                ::config::setKey fsvideosrc ""
+                ::config::setKey fsvideosrcdev ""
+
+                ::config::setKey fs_in_volume -5.0
+                ::config::setKey fs_out_volume -5.0
+
+
+		::config::setKey expanded_group_offline 0
+		::config::setKey expanded_group_nonim 0
+
 		#Advanced options, not in preferences window
 		# Create the entry in the list and then, set
 		# the variable at bottom
@@ -311,7 +365,14 @@ namespace eval ::config {
 			[list local tooltips bool tooltips] \
 			[list local emailsincontactlist bool emailsinlist] \
 			[list local leavejoinsinchat bool leavejoinsinchat] \
-			[list local animatenotify bool animatenotify] \
+			[list local animatenotify bool animatenotify]\
+	        ]
+
+                if { ![OnWin] } {
+                        lappend advanced_options [list local blinktray bool blinktray]
+                }
+
+                lappend advanced_options \
 			[list local enablebanner bool showbanner] \
 			[list local truncatenames bool truncatenames1] \
 			[list local truncatenicks bool truncatenames2] \
@@ -330,6 +391,8 @@ namespace eval ::config {
 			[list local show_login_screen_links bool showlinksinlogin] \
 			[list local big_incoming_smileys bool noresizesmileys] \
 			[list local old_dpframe bool showonedpframe] \
+			[list local contentroaming bool contentroamingsetting] \
+			[list local show_not_in_list bool shownotinlist] \
 			[list title notifyoffset] \
 			[list local notifyXoffset int xoffset] \
 			[list local notifyYoffset int yoffset] \
@@ -341,15 +404,16 @@ namespace eval ::config {
 			[list local notifyemail bool notify3] \
 			[list local notifyemailother bool notify4] \
 			[list local notifytimeout int notifytimeout] \
+			[list local no_blocked_notif bool no_blocked_notif] \
 			[list local soundactive bool soundactive] \
 		        [list local autolisten_voiceclips bool autolisten_voiceclips] \
 			[list local recentmsg bool recentmsg] \
 			[list title connection] \
 			[list local getdisppic bool getdisppic] \
 			[list local checkemail bool checkemail] \
-			[list local default_ns_server str notificationserver]\
 			[list local lazypicretrieval bool lazypicretrieval]\
 			[list local noftpreview bool noftpreview]\
+			[list local getprofileoffline bool getprofileoffline] \
 			[list title MSN] \
 			[list local displayp4context bool displayp4context] \
 			[list local p4contextprefix str p4contextprefix] \
@@ -364,9 +428,150 @@ namespace eval ::config {
 			[list local globaloverride bool globaloverride ] \
 			[list local escape_close_cw bool escapeclosescw ] \
 			[list local no_oim_confirmation bool nooimconfirmation ] \
-			[list global disableprofiles bool disableprofiles] \
-		]
+			[list global disableprofiles bool disableprofiles]
 
+		set osspecific_keys [list receiveddir soundcommand browser notifyXoffset \
+					notifyYoffset filemanager openfilecommand usesnack \
+					wingeometry backgroundcolor dockbounce newchatwinstate \
+					newmsgwinstate psmplace mailcommand \
+					webcamDevice snackInputDevice snackMixerDevice snackOutputDevice]
+
+		set locale_codes [list \
+			[list af 1078] \
+			[list ar-ae 14337] \
+			[list ar-bh 15361] \
+			[list ar-dz 5121] \
+			[list ar-eg 3073] \
+			[list ar-iq 2049] \
+			[list ar-jo 11265] \
+			[list ar-kw 13313] \
+			[list ar-lb 12289] \
+			[list ar-ly 4097] \
+			[list ar-ma 6145] \
+			[list ar-om 8193] \
+			[list ar-qa 16385] \
+			[list ar-sa 1025] \
+			[list ar-sy 10241] \
+			[list ar-tn 7169] \
+			[list ar-ye 9217] \
+			[list az-az 1068] \
+			[list az-az 2092] \
+			[list be 1059] \
+			[list bg 1026] \
+			[list ca 1027] \
+			[list cs 1029] \
+			[list da 1030] \
+			[list de-de 1031] \
+			[list de-at 3079] \
+			[list de-li 5127] \
+			[list de-lu 4103] \
+			[list de-ch 2055] \
+			[list el 1032] \
+			[list en-au 3081] \
+			[list en-bz 10249] \
+			[list en-ca 4105] \
+			[list en-cb 9225] \
+			[list en-gb 2057] \
+			[list en-ie 6153] \
+			[list en-jm 8201] \
+			[list en-nz 5129] \
+			[list en-ph 13321] \
+			[list en-za 7177] \
+			[list en-tt 11273] \
+			[list en-gb 2057] \
+			[list en-us 1033] \
+			[list es-es 1034] \
+			[list es-ar 11274] \
+			[list es-bo 16394] \
+			[list es-cl 13322] \
+			[list es-co 9226] \
+			[list es-cr 5130] \
+			[list es-do 7178] \
+			[list es-ec 12298] \
+			[list es-gt 4106] \
+			[list es-hn 18442] \
+			[list es-mx 2058] \
+			[list es-ni 19466] \
+			[list es-pa 6154] \
+			[list es-pe 10250] \
+			[list es-pr 20490] \
+			[list es-py 15370] \
+			[list es-sv 17418] \
+			[list es-uy 14346] \
+			[list es-ve 8202] \
+			[list et 1061] \
+			[list eu 1069] \
+			[list fa 1065] \
+			[list fi 1035] \
+			[list fo 1080] \
+			[list fr-fr 1036] \
+			[list fr-be 2060] \
+			[list fr-ca 3084] \
+			[list fr-lu 5132] \
+			[list fr-ch 4108] \
+			[list gd-ie 2108] \
+			[list gd 1084] \
+			[list he 1037] \
+			[list hi 1081] \
+			[list hr 1050] \
+			[list hu 1038] \
+			[list hy 1067] \
+			[list is 1039] \
+			[list id 1057] \
+			[list it-it 1040] \
+			[list it-ch 2064] \
+			[list ja 1041] \
+			[list ko 1042] \
+			[list lv 1062] \
+			[list lt 1063] \
+			[list mk 1071] \
+			[list ms-my 1086] \
+			[list ms-bn 2110] \
+			[list mt 1082] \
+			[list mr 1102] \
+			[list nl-nl 1043] \
+			[list nl-be 2067] \
+			[list no-nb 1044] \
+			[list no-no 2068] \
+			[list pl 1045] \
+			[list pt-pt 2070] \
+			[list pt-br 1046] \
+			[list rm 1047] \
+			[list ro 1048] \
+			[list ro-mo 2072] \
+			[list ru 1049] \
+			[list ru-mo 2073] \
+			[list sa 1103] \
+			[list sb 1070] \
+			[list sl 1060] \
+			[list sk 1051] \
+			[list sq 1052] \
+			[list sr-sp 3098] \
+			[list sr-sp 2074] \
+			[list st 1072] \
+			[list sv-se 1053] \
+			[list sv-fi 2077] \
+			[list sw 1089] \
+			[list ta 1097] \
+			[list th 1054] \
+			[list tn 1074] \
+			[list tr 1055] \
+			[list ts 1073] \
+			[list tt 1092] \
+			[list uk 1058] \
+			[list ur 1056] \
+			[list uz-uz 2115] \
+			[list uz-uz 1091] \
+			[list vi 1066] \
+			[list xh 1076] \
+			[list yi 1085] \
+			[list zu 1077] \
+			[list zh-cn 2052] \
+			[list zh-hk 3076] \
+			[list zh-mo 5124] \
+			[list zh-sg 4100] \
+			[list zh-tw 1028] \
+		]
 	}
 
 	proc globalDefaults {} {
@@ -377,6 +582,7 @@ namespace eval ::config {
 		setGlobalKey skin "default"			;#AMSN skin
 		setGlobalKey disableprofiles 0 ;#Disable profiles (useful for cybercafes or similar)
 
+
 		#Specific configs for Mac OS X (Aqua) first, and for others systems after
 		if { [OnMac] } {
 			setGlobalKey basefont [list {Lucida Grande} 12 normal]	;#AMSN Mac OS X base font
@@ -385,6 +591,84 @@ namespace eval ::config {
 		} else {
 			setGlobalKey basefont [list Helvetica 11 normal]	;#AMSN base font
 		}
+		setGlobalKey dpi 0	;# The screen's DPI if we want to force it. 0 to let it use default
+
+	}
+
+	proc getDefaultDownloadDir { } {
+	
+		#Dir for received files
+		if { [OnDarwin] } {
+		        return "[file join $::env(HOME) Desktop]"
+		} elseif { [OnWin] } {
+			if {[info exists env(USERPROFILE)]} {
+				return "[file join $::env(USERPROFILE) amsn_received]"
+			} else {
+				return "[file join [pwd] amsn_received]"
+			}
+		} elseif { [OnUnix] } {
+			set path ""
+			if {[catch {
+				if {[info exists ::env(XDG_CONFIG_HOME)] } {
+					set config_dir [set ::env(XDG_CONFIG_HOME)]
+				} else {
+					set config_dir [file join [set ::env(HOME)] .config]
+				}
+				status_log "Opening xdg file in [file join $config_dir user-dirs.dir]"
+				set fd [open [file join $config_dir user-dirs.dirs] r]
+				set data [read $fd]
+				close $fd
+				foreach line [split $data "\n"] {
+					status_log "Parsing line $line"
+					set line [string trim $line]
+					if {[string first "XDG_DOWNLOAD_DIR" $line ] == 0} {
+						status_log "found XDG_DOWNLOAD_DIR"
+						set line [string trim [string range $line 16 end]]
+						if {[string first "=" $line] == 0} {
+							status_log "Found ="
+							set path [string trim [string range $line 1 end]]
+							if {[string first "\"" $path] == 0} {
+								status_log "found first quote"
+								set path [string trim [string range $path 1 end]]
+								set second_quote [string first "\"" $path]
+								status_log "second quote at $second_quote"
+								if {$second_quote != -1} {
+									incr second_quote -1
+									set path [string trim [string range $path 0 $second_quote]]
+									status_log "path is $path"
+								}
+								break;
+							}
+						}
+					}
+				}
+				if {$path != ""} {
+					status_log "Found a path : $path"
+					if {[string first "\$HOME/" $path] == 0} {
+						status_log "Path is relative"
+						set relative 1
+						set path [string range  $path 6 end]
+						set path [file join $::env(HOME) $path]
+					}
+					status_log "Final path is : $path"
+				}
+			}]} {
+				set path ""
+				status_log "Error parsing XDG_DOWNLOAD_DIR" red
+			}
+			if {$path == ""} {
+				if { [OnMaemo] } {
+					return "[file join $::env(HOME) MyDocs amsn_received]"
+				}  else { 
+					return "[file join $::env(HOME) amsn_received]"
+				}
+			} else {
+				return $path
+			}
+			
+		} else {
+			return "[file join [pwd] amsn_received]"
+		}
 	}
 
 	proc get {key} {
@@ -416,18 +700,39 @@ namespace eval ::config {
 		set ::config($key) $value
 	}
 
+    proc searchKeys { search_key } {
+        set r [list]
+        foreach key [::config::getKeys] {
+            # non case-sensitive string matching
+            if { [string match -nocase "*${search_key}*" $key] } {
+                # They key matches, so append it to the return var.
+                lappend r $key
+            }
+        }
+        return $r
+    }
+
 	proc setAll {values} {
 		array set ::config $values
 	}
 
 	proc unsetKey {key} {
-		unset ::config($key)
+		if { [::config::isSet $key] } {
+			unset ::config($key)
+		}
 	}
 	
 	proc isSet {key} {
 		return [info exists ::config($key)]
 	}
 
+	proc renameKey {srcKey dstKey} {
+		if {[info exists ::config($srcKey)]} {
+			set ::config($dstKey) $::config($srcKey)
+			unset ::config($srcKey)
+		}
+	}
+
 	proc getGlobalKey {key {default ""}} {
 		if { [info exists ::gconfig($key)] } {
 			return [set ::gconfig($key)]
@@ -525,7 +830,7 @@ namespace eval ::config {
 }
 
 proc save_config {} {
-	global HOME HOME2 version password custom_emotions
+	global HOME HOME2 version password custom_emotions osspecific_keys
 	
 	#saving the plugins
 	::plugins::save_config
@@ -539,6 +844,7 @@ proc save_config {} {
 			set file_id [open "[file join ${HOME} config.xml.temp]" w]
 		}
 	} res]} {
+		msg_box "[trans configpermissionerror ${HOME}] : \n$res"
 		return 0
 	}
 	fconfigure $file_id -encoding utf-8
@@ -553,13 +859,25 @@ proc save_config {} {
 		set password ""
 	}
 
+	# rename os-specific keys
+	foreach srcKey $osspecific_keys {
+		set dstKey "${srcKey}_[::config::getKey os]"
+		::config::renameKey $srcKey $dstKey
+	}
+
 	#Start of config file
 	puts $file_id  "<?xml version=\"1.0\"?>\n\n<config>"
 
 	#Save all keys except special ones
 	foreach var_attribute [::config::getKeys] {
 		set var_value [::config::getKey $var_attribute]
-		if { "$var_attribute" != "remotepassword" } {
+		
+		#Convert absolute path to relative path
+		if { "$var_attribute" == "displaypic" } {
+			set var_value [PathAbsToRel $var_value]
+		}
+
+		if { ("$var_attribute" != "remotepassword") && ("$var_attribute" != "os") && ([string first "tempgroup_" "$var_attribute"] != 0) } {
 			set var_value [::sxml::xmlreplace $var_value]
 			puts $file_id "   <entry>\n      <attribute>$var_attribute</attribute>\n      <value>$var_value</value>\n   </entry>"
 		}
@@ -568,13 +886,13 @@ proc save_config {} {
 	#Save encripted password
 	if { ([::config::getKey save_password]) && ($password != "")} {
 		set key [string range "${loginback}dummykey" 0 7]
-		binary scan [::des::encrypt $key "${password}\n"] h* encpass
+		binary scan [::DES::des -mode ecb -dir encrypt -key $key -- "${password}\n"] h* encpass
 		puts $file_id "   <entry>\n      <attribute>encpassword</attribute>\n      <value>$encpass</value>\n   </entry>"
 	}
 
 	#Save encripted remote password
 	set key [string range "${loginback}dummykey" 0 7]
-	binary scan [::des::encrypt $key "[::config::getKey remotepassword]\n"] h* encpass
+	binary scan [::DES::des -mode ecb -dir encrypt -key $key -- "[::config::getKey remotepassword]\n"] h* encpass
 	puts $file_id "   <entry>\n      <attribute>remotepassword</attribute>\n      <value>$encpass</value>\n   </entry>\n"
 
 	#Save custom emoticons
@@ -582,7 +900,13 @@ proc save_config {} {
 		puts $file_id "   <emoticon>"
 		array set emotion $custom_emotions($name)
 		foreach attribute [array names emotion] {
-			set value [::sxml::xmlreplace $emotion($attribute)]
+			set tmp_value $emotion($attribute)
+			#Convert absolute paths to subdirs in the profile-home to relative paths
+			if { $attribute == "file" || $attribute == "sound" } {
+				set tmp_value [PathAbsToRel $tmp_value]
+			}
+
+			set value [::sxml::xmlreplace $tmp_value]
 			set attribute [::sxml::xmlreplace $attribute]
 			#set var_attribute [::sxml::xmlreplace [string map [list "${custom}_" ""] $attribute ]]
 			#set var_value [::sxml::xmlreplace $emotions($attribute)]
@@ -597,7 +921,17 @@ proc save_config {} {
 	close $file_id
 	
 	#writing was successful, so move config.xml.temp to config.xml
-	file rename -force [file join ${HOME} config.xml.temp] [file join ${HOME} config.xml]
+	catch {file delete -force [file join ${HOME} config.xml]}
+	if {[catch {file copy -force [file join ${HOME} config.xml.temp] [file join ${HOME} config.xml]} err] } {
+		msg_box "[trans configpermissionerror ${HOME}] : \n$err"
+	}
+	catch {file delete -force [file join ${HOME} config.xml.temp]}
+
+	# re-rename os-specific keys
+	foreach dstKey $osspecific_keys {
+		set srcKey "${dstKey}_[::config::getKey os]"
+		::config::renameKey $srcKey $dstKey
+	}
 
 	::config::setKey login $loginback
 	set password $passback
@@ -617,7 +951,7 @@ proc new_config_entry  {cstack cdata saved_data cattr saved_attr args} {
 }
 
 proc load_config {} {
-	global HOME password protocol
+	global HOME password osspecific_keys
 
 	#Create custom smileys folder
 	create_dir "[file join ${HOME} smileys]"
@@ -639,6 +973,13 @@ proc load_config {} {
 			sxml::register_routine $file_id "config:emoticon" "::smiley::newCustomEmoticonXML"
 			set val [sxml::parse $file_id]
 			sxml::end $file_id
+
+			# rename os-specific keys
+			foreach dstKey $osspecific_keys {
+				set srcKey "${dstKey}_[::config::getKey os]"
+				::config::renameKey $srcKey $dstKey
+			}
+
 			status_log "load_config: Config loaded\n" green
 			if { [winfo exists .smile_selector]} { destroy .smile_selector }
 
@@ -687,7 +1028,7 @@ proc load_config {} {
 		set key [string range "[::config::getKey login]dummykey" 0 7]
 		set password [::config::getKey encpassword]
 		catch {set encpass [binary format h* [::config::getKey encpassword]]}
-		catch {set password [::des::decrypt $key $encpass]}
+		catch {set password [::DES::des -mode ecb -dir decrypt -key $key -- $encpass]}
 		#puts "Password length is: [string first "\n" $password]\n"
 		set password [string range $password 0 [expr { [string first "\n" $password] -1 }]]
 		#puts "Password is: $password\nHi\n"
@@ -698,19 +1039,12 @@ proc load_config {} {
 	if {[::config::getKey remotepassword]!=""} {
 		set key [string range "[::config::getKey login]dummykey" 0 7]
 		catch {set encpass [binary format h* [::config::getKey remotepassword]]}
-		catch {::config::setKey remotepassword [::des::decrypt $key $encpass]}
+		catch {::config::setKey remotepassword [::DES::des -mode ecb -dir decrypt -key $key -- $encpass]}
 		#puts "Password length is: [string first "\n" [::config::getKey remotepassword]]\n"
 		::config::setKey remotepassword [string range [::config::getKey remotepassword] 0 [expr { [string first "\n" [::config::getKey remotepassword]] -1 }]]
 		#puts "Password is: [::config::getKey remotepassword]\nHi\n"
 	}
 
-	::config::setKey clientid 0
-	::config::setKey protocol 11
-	::MSN::setClientCap msnc4
-	::MSN::setClientCap inkgif
-	::MSN::setClientCap multip
-	::MSN::setClientCap voice
-
 	# Load up the personal states
 	LoadStateList
 	if { [winfo exists .my_menu] } {CreateStatesMenu .my_menu}
@@ -754,6 +1088,9 @@ proc load_config {} {
 	# We now want to force the NAT Keep alive to enabled
 	::config::setKey keepalive 1
 
+	# Disable MSN spaces - disabled in the servers
+	::config::setKey showspaces 0
+
 	# We loaded the config, we will now load the DP pictures.. 
 	load_my_pic
 	
@@ -841,18 +1178,22 @@ proc LoadLoginList {{trigger 0}} {
 		set temp_data [split $tmp_data]
 
 		set user_login [lindex $tmp_data 0]
+		set dirname [string map {"@" "_" "." "_"} $user_login]
 		set locknum [lindex $tmp_data 1]
 		if {[string first "@" $user_login] == -1} {
 			status_log "Profile $user_login is wrong!! Fixing\n" red
 			#Profile is wrong, fix it from config file
 			set oldHOME $HOME
-			set dirname [string map {"@" "_" "." "_"} $user_login]
-			set HOME [file join $HOMEE $dirname]
+			set HOME [file join $HOME2 $dirname]
 			load_config
 			set user_login [::config::getKey login]
 			set HOME $oldHOME
 		}
 
+		if {![file exists [file join $HOME2 $dirname]] || ![file isdirectory [file join $HOME2 $dirname]] || ![file exists [file join $HOME2 $dirname config.xml]]} {
+			#skip login if profile dir doesn't exist or config.xml is missing
+			continue
+		}
 
 #	    puts "temp data : $temp_data"
 		if { $locknum == "" } {
@@ -1080,7 +1421,7 @@ proc ConfigChange { window email } {
 				if { [CheckLock $email] == -1 } {
 					status_log "ConfigChange: Profile is locked\n" blue
 
-					msg_box [trans profileinuse]
+					::amsn::messageBox [trans profileinuse] ok info "aMSN"
 					set HOME $OLDHOME
 
 					# Reselect previous element in combobox
@@ -1163,6 +1504,7 @@ proc SwitchToDefaultProfile { } {
 	::config::setKey save_password 0
 	::config::setKey connectas $connectas
 	::config::setKey keep_logs 0
+	::config::setKey webcamlogs 0
 	::config::setKey log_event_connect 0
 	::config::setKey log_event_disconnect 0
 	::config::setKey log_event_email 0
@@ -1170,6 +1512,14 @@ proc SwitchToDefaultProfile { } {
 	::config::setKey log_event_nick 0
 	::config::setKey log_event_psm 0
 	::config::setKey displaypic "amsn.png"
+	::config::setKey receiveddir [::config::getDefaultDownloadDir]
+
+	foreach file [glob -nocomplain -type f [file join $::HOME2 displaypic *]] {
+		catch {file delete $file}
+	}
+	foreach file [glob -nocomplain -type f [file join $::HOME2 displaypic cache *]] {
+		catch {file delete $file}
+	}
 }
 
 #///////////////////////////////////////////////////////////////////////////////
@@ -1245,35 +1595,73 @@ proc CreateProfile { email } {
 	#set dirname [split $email "@ ."]
 	#set dirname [join $dirname "_"]
 	set HOME "[file join $HOME2 $dirname]"
-	create_dir $HOME
-	set log_dir "[file join ${HOME} logs]"
-	create_dir $log_dir
-	set webcam_dir "[file join ${HOME} webcam]"
-	create_dir $webcam_dir
+	if {![file exists $HOME] || ![file exists [file join $HOME config.xml]]} {
+		status_log "profile doesn't exist -> create it!" blue
 
-	# Load default config initially
-	# file copy -force [file join $HOME2 config.xml] $newHOMEdir
+		create_dir $HOME
+		set log_dir "[file join ${HOME} logs]"
+		create_dir $log_dir
+		set webcam_dir "[file join ${HOME} webcam]"
+		create_dir $webcam_dir
+		set displaypic_dir "[file join ${HOME} displaypic]"
+		create_dir $displaypic_dir
 
-	set oldpassword $password
-	set oldsavepwd [::config::getKey save_password]
-	set oldautoconnect [::config::getKey autoconnect 0]
+		# Load default config initially
+		# file copy -force [file join $HOME2 config.xml] $newHOMEdir
 
-	# set all config keys to default
-	::config::configDefaults
+		set oldpassword $password
+		set oldsavepwd [::config::getKey save_password]
+		set oldautoconnect [::config::getKey autoconnect 0]
 
-	# re-set login, password an remember password value before saving
-	::config::setKey login $email
-	set password $oldpassword
-	::config::setKey save_password $oldsavepwd
-	::config::setKey autoconnect $oldautoconnect
-	save_config
+		# set all config keys to default
+		::config::configDefaults
 
-	unset oldpassword
-	unset oldsavepwd
-	unset oldautoconnect
+		# re-set login, password an remember password value before saving
+		::config::setKey login $email
+		set password $oldpassword
+		::config::setKey save_password $oldsavepwd
+		::config::setKey autoconnect $oldautoconnect
+		save_config
+
+		unset oldpassword
+		unset oldsavepwd
+		unset oldautoconnect
+	} else {
+		status_log "profile already exists -> skip creating!" blue
+
+		set oldpassword $password
+		set oldsavepwd [::config::getKey save_password]
+		set oldautoconnect [::config::getKey autoconnect 0]
+
+		# set all config keys to default
+		::config::configDefaults
+
+		#load settings
+		load_config
+
+		set password $oldpassword
+		::config::setKey save_password $oldsavepwd
+		::config::setKey autoconnect $oldautoconnect
+		save_config
+
+		unset oldpassword
+		unset oldsavepwd
+		unset oldautoconnect
+	}
 
 	# Add to login list
 	LoginList add 0 $email 0
+
+	# Make sure we delete old lock
+	if { [info exists lockSock] } {
+		if { $lockSock != 0 } {
+			close $lockSock
+			unset lockSock
+		}
+	}
+
+	#Lock profile
+	LockProfile $email
 	SaveLoginList
 
 	# Fire event
@@ -1369,18 +1757,17 @@ proc CheckLock { email } {
 	set Port [LoginList getlock 0 $email]
 	status_log "CheckLock: LoginList getlock called. Lock=$Port\n" blue
 	if { $Port != 0 } {
-		if { [OnWinVista] || [catch {socket -server phony $Port} newlockSock] != 0 } {
+		if { [catch {socket -server phony $Port} newlockSock] != 0 } {
 			#status_log "CheckLock Port is already in use: $newlockSock\n" red
 			# port is taken, let's make sure it's a profile lock
-			foreach {local_host} { localhost "[info hostname]" 127.0.0.1 } {
+			foreach {local_host} [list localhost [info hostname] 127.0.0.1] {
 				if {[catch {socket $local_host $Port} clientSock] == 0 } {
-					set done 1
 					status_log "CheckLock: Can connect to port. Sending PING\n" blue
 					fileevent $clientSock readable "lockcltHdl $clientSock"
-					fconfigure $clientSock -buffering line
+					fconfigure $clientSock -buffering line -blocking 0
 					puts $clientSock "AMSN_LOCK_PING"
-					after 5000 [list set response failed]
-					vwait response
+					after 200 [list set response failed]
+					tkwait variable response
 
 					if { $response == "AMSN_LOCK_PONG" } {
 						status_log "CheckLock: Got PONG response\n" green
@@ -1395,7 +1782,7 @@ proc CheckLock { email } {
 						# other non amsn program is using the lock port, we better reset the lock to 0
 						LoginList changelock 0 $email 0
 					}
-				break
+					break
 				}
 			}
 		} else {
@@ -1445,12 +1832,8 @@ proc LockProfile { email } {
 	while { $tries < 6 } {
 		set Port [GetRandomProfilePort]
 		status_log "LockProfile: Got random port $Port\n" blue
-		if { [::config::getKey enableremote] == 1 } {
-			set cmd "socket -server lockSvrNew $Port"
-		} else {
-			set cmd "socket -myaddr 127.0.0.1 -server lockSvrNew $Port"
-		}
-		if { [catch {eval $cmd} newlockSock] == 0  } {
+
+		if { [catch {socket -server lockSvrNew $Port} newlockSock] == 0  } {
 			LoginList changelock 0 $email $Port
 			set lockSock $newlockSock
 			break
@@ -1471,10 +1854,10 @@ proc LockProfile { email } {
 proc lockSvrNew { sock addr port} {
 	status_log "lockSvrNew: Accepting connection on port $port\n" blue
 
-#	if { $addr == "127.0.0.1" } {
+	if {[::config::getKey enableremote] || $addr == "127.0.0.1" } {
 		fileevent $sock readable "lockSvrHdl $addr $sock"
 		fconfigure $sock -buffering line
-#	}
+	}
 }
 
 proc lockSvrHdl { addr sock } {
@@ -1563,6 +1946,38 @@ proc create_dir {path} {
 }
 #///////////////////////////////////////////////////////////////////////
 
+proc PathAbsToRel { path } {
+	global HOME
+	
+	if { [string index $path 0] == "~" && $path != [PathRelToAbs $path] } {
+		set path [PathRelToAbs $path]
+	}
+
+	#Convert absolute path to a relative path
+	if { [string length $path] > [string length $HOME] && [string equal -length [string length $HOME] $HOME $path] && [string index $path [string length $HOME]] == "/"} {
+		set path "[string range $path [expr [string length $HOME] + 1] end]"
+	}
+
+	return $path
+}
+
+proc PathRelToAbs { path } {
+	global HOME
+	
+	#Expand relative path to an absolute path again
+	if { [string index $path 0] == "~" } {
+		set path2 "$HOME[string range $path 1 end]"
+		if {[file exists $path2]} {
+			return $path2
+		}
+	}
+	
+	if {[file exists [file join $HOME $path]]} {
+		return [file join $HOME $path]
+	}
+	return $path
+}
+
 if { $initialize_amsn == 1 } {
 	###############################################################
 	create_dir $HOME
@@ -1592,4 +2007,31 @@ if { $initialize_amsn == 1 } {
 
 	# Init skin and smileys
 	::skin::reloadSkin [::config::getGlobalKey skin]
+
+	set machineguid [::config::getGlobalKey machineguid ""]
+	if {$machineguid == "" } {
+		set machineguid "{"
+		for {set i 0} {$i < 8} {incr i} {
+			append machineguid [format %x [expr {int(rand() * 16)}]]
+		}
+		append machineguid "-"
+		for {set i 0} {$i < 4} {incr i} {
+			append machineguid [format %x [expr {int(rand() * 16)}]]
+		}
+		append machineguid "-"
+		for {set i 0} {$i < 4} {incr i} {
+			append machineguid [format %x [expr {int(rand() * 16)}]]
+		}
+		append machineguid "-"
+		for {set i 0} {$i < 4} {incr i} {
+			append machineguid [format %x [expr {int(rand() * 16)}]]
+		}
+		append machineguid "-"
+		for {set i 0} {$i < 12} {incr i} {
+			append machineguid [format %x [expr {int(rand() * 16)}]]
+		}
+		append machineguid "}"
+
+		::config::setGlobalKey machineguid $machineguid
+	}
 }
