@@ -3,7 +3,7 @@
 #
 ################################################################################
 
-::Version::setSubversionId {$Id: loging.tcl 9146 2007-12-23 15:48:09Z kakaroto $}
+::Version::setSubversionId {$Id: loging.tcl 11690 2009-10-11 23:22:07Z kakaroto $}
 
 # TODO Implement some sort of log file size limit or date limit (remove any log entries older than date)
 # TODO Save to LOG (if logging disabled, allows to log certain conversations only)
@@ -26,12 +26,15 @@ namespace eval ::log {
 		if { [LoginList exists 0 [::config::getKey login]] == 0 } {
 			LogArray $email set 0
 		} else {
-			LogArray $email set [CheckLogDate $email]
+			if {[catch {set fd [CheckLogDate $email]}] } {
+				return
+			}
+			LogArray $email set $fd
 
 			if { [::config::getKey lineflushlog] == 1 } {
-				fconfigure [LogArray $email get] -buffering none -encoding utf-8 
+				fconfigure $fd -buffering none -encoding utf-8 
 			} else {
-				fconfigure [LogArray $email get] -buffersize 1024 -encoding utf-8
+				fconfigure $fd -buffersize 1024 -encoding utf-8
 			}
 		}
 	}
@@ -417,26 +420,22 @@ namespace eval ::log {
 		global log_dir langenc logvar
 
 		# Get all the contacts with logs
-		set lDirs [concat ${log_dir} [glob -nocomplain -types d "${log_dir}/*"]]
+		set lDirs [concat [list ${log_dir}] [glob -nocomplain -types d "${log_dir}/*"]]
 
+		set contact_list [list]
 		foreach sDir $lDirs {
 			foreach sLogFile [glob -tails -nocomplain -types f -directory ${sDir} "*.log"] {
 				set sLogFile [ string range $sLogFile 0 [ expr { [string length $sLogFile] - 5 } ] ]
-				set hNames($sLogFile) 1
+				lappend contact_list $sLogFile
 			}
 		}
-		
-		set contact_list [ array names hNames ]
 
-		#Sorts contacts
-		set sortedcontact_list [lsort -dictionary $contact_list]
-
-		#Add the eventlog
-		lappend sortedcontact_list eventlog
+		#Sort contacts
+		set contact_list [lsort -dictionary -unique $contact_list]
 
 		#If there is no email defined, we replace it by the first email in the dictionary order
 		if {$email == ""} {
-			set email [lindex $sortedcontact_list 0]
+			set email [lindex $contact_list 0]
 		}
 		
 		set fileid [LogArray $email get]
@@ -447,11 +446,15 @@ namespace eval ::log {
 
 		set wname [::log::wname $email]
 
-		if { [catch {toplevel ${wname} -width 600 -height 400 -borderwidth 0 -highlightthickness 0 } res ] } {
-			raise ${wname}
-			focus ${wname}
-			wm deiconify ${wname}
+		if { [winfo exists $wname] } {
+		    raise ${wname}
+		    focus ${wname}
+		    wm deiconify ${wname}
+		    return 0
+		} else {
+		    if {[catch {toplevel ${wname} -width 600 -height 400 -borderwidth 0 -highlightthickness 0 } res ] } {
 			return 0
+		    }
 		}
 		
 		wm group ${wname} .
@@ -497,9 +500,9 @@ namespace eval ::log {
 		}
 
 		frame $wname.top.contact -borderwidth 0
-		combobox::combobox $wname.top.contact.list -editable true -highlightthickness 0 -width 22
+		combobox::combobox $wname.top.contact.list -highlightthickness 0 -width 22
 		$wname.top.contact.list list delete 0 end
-		foreach contact $sortedcontact_list {
+		foreach contact $contact_list {
 			$wname.top.contact.list list insert end $contact
 		}
 
@@ -509,35 +512,28 @@ namespace eval ::log {
 		set exactMatch [lsearch -exact $list $email]
 		#Select the email in the list when we open the window with the result of the search
 		$wname.top.contact.list select $exactMatch
-		$wname.top.contact.list configure -command "::log::ChangeLogWin $wname $email"
+		$wname.top.contact.list configure -command [list ::log::ChangeLogWin $wname]
 		$wname.top.contact.list configure -editable false
 
 		pack $wname.top.contact.list -side left
 		pack $wname.top.contact -side left
 
-		::log::LogsByDate $wname $email "1"
+		::log::LogsByDate $wname $email
 
 		ParseLog $wname $logvar
 
-		button $wname.buttons.close -text "[trans close]" -command "destroy $wname"
-		button $wname.buttons.find -text "[trans find]" -command "$wname.search show"
+		button $wname.buttons.close -text "[trans close]" -command [list destroy $wname]
+		button $wname.buttons.find -text "[trans find]" -command [list $wname.search show]
 		button $wname.buttons.stats -text "[trans stats]" -command "::log::stats"
-		button $wname.buttons.save -text "[trans savetofile]" -command "::log::SaveToFile ${wname} ${email} [list ${logvar}]"
-		button $wname.buttons.clear -text "[trans clearlog]" \
-		    -command "if { !\[winfo exists $wname.top.date.list\] } { \
-				                    set date \".\" \
-				              } else {
-				                    set date \[$wname.top.date.list list get \[$wname.top.date.list curselection\]\]\
-					      }
-                                              if { \[::log::ClearLog $email \"\$date\"\] } { 
-				                    destroy $wname
-			         	      }" \
+		button $wname.buttons.save -text "[trans savetofile]" -command [list ::log::SaveToFile ${wname}]
+		button $wname.buttons.clear -text "[trans clearlog]"  -command [list ::log::ClearLogGUI ${wname}]
 		    
 
 
 		menu ${wname}.copypaste -tearoff 0 -type normal
-		${wname}.copypaste add command -label [trans copy] -command "tk_textCopy ${wname}.blueframe.log.txt"
+		${wname}.copypaste add command -label [trans copy] -command [list tk_textCopy ${wname}.blueframe.log.txt]
 		
+		pack $wname.buttons -side bottom -fill x -pady 3
 		pack $wname.top -side top -fill x
 		pack $wname.blueframe.log.ys -side right -fill y
 		pack $wname.blueframe.log.txt -side left -expand true -fill both
@@ -548,7 +544,6 @@ namespace eval ::log {
 		pack $wname.buttons.save -padx 0 -side right
 		pack $wname.buttons.clear -padx 0 -side right
 		pack $wname.buttons.find -padx 0 -side right
-		pack $wname.buttons -side bottom -fill x -pady 3
 		bind $wname <<Escape>> "destroy $wname"
 		bind ${wname}.blueframe.log.txt <<Button3>> "tk_popup ${wname}.copypaste %X %Y"
 		moveinscreen $wname 30
@@ -559,33 +554,50 @@ namespace eval ::log {
 		global webcam_dir langenc logvar
 
 		# Get all the contacts with saved webcam sessions
-		set lDirs [concat ${webcam_dir} [glob -nocomplain -types d "${webcam_dir}/*"]]
+		set lDirs [concat [list ${webcam_dir}] [glob -nocomplain -types d "${webcam_dir}/*"]]
 
+		set contact_list [list]
 		foreach sDir $lDirs {
-			foreach sLogFile [glob -tails -nocomplain -types f -directory ${sDir} "*.cam"] {
+			foreach sLogFile [glob -tails -nocomplain -types f -directory ${sDir} "*.cam" "*.CAM"] {
+
+				#it's possible that if we move the cam files in another file system and then we import them, every extensions can be uppercased. So we have to convert *.CAM in *.cam. The same for dat files.
+				if {[file extension $sLogFile] eq ".CAM"} {
+					set filenoext "${sDir}/"
+					append filenoext [string range $sLogFile 0 end-4]
+					catch { [file rename "${filenoext}.CAM" "${filenoext}.cam"]}
+					catch { [file rename "${filenoext}.DAT" "${filenoext}.dat"]}
+				}
+
 				set sLogFile [ string range $sLogFile 0 [ expr { [string length $sLogFile] - 5 } ] ]
-				set hNames($sLogFile) 1
+				lappend contact_list $sLogFile
 			}
 		}
 
-		set contact_list [ array names hNames ]
+		if { [llength $contact_list] == 0 } {
+			::amsn::infoMsg "[trans nologfile [::config::getKey login]]"
+			return
+		}
 		
-		set sortedcontact_list [lsort -dictionary $contact_list]
+		set contact_list [lsort -dictionary -unique $contact_list]
 
 		#If there is no email defined, we replace it by the first email in the dictionary order
 		if {$email == ""} {
-			set email [lindex $sortedcontact_list 0]
+			set email [lindex $contact_list 0]
 		}
 		
 		set wname [::log::cam_wname $email]
 
-		if { [catch {toplevel ${wname} -borderwidth 0 -highlightthickness 0 } res ] } {
-			raise ${wname}
-			focus ${wname}
-			wm deiconify ${wname}
+		if { [winfo exists $wname] } {
+		    raise ${wname}
+		    focus ${wname}
+		    wm deiconify ${wname}
+		    return 0
+		} else {
+		    if { [catch {toplevel ${wname} -borderwidth 0 -highlightthickness 0 } res ] } {
 			return 0
+		    }
 		}
-		
+
 		wm group ${wname} .
 
 		if { [file exists [file join ${webcam_dir} ${email}.cam]] } {
@@ -616,7 +628,7 @@ namespace eval ::log {
 		frame $wname.top.contact -borderwidth 0
 		combobox::combobox $wname.top.contact.list -editable true -highlightthickness 0 -width 22
 		$wname.top.contact.list list delete 0 end
-		foreach contact $sortedcontact_list {
+		foreach contact $contact_list {
 			$wname.top.contact.list list insert end $contact
 		}
 
@@ -626,38 +638,34 @@ namespace eval ::log {
 		set exactMatch [lsearch -exact $list $email]
 		#Select the email in the list when we open the window with the result of the search
 		$wname.top.contact.list select $exactMatch
-		$wname.top.contact.list configure -command "::log::ChangeCamLogWin $wname $email"
+		$wname.top.contact.list configure -command [list ::log::ChangeCamLogWin $wname ]
 		$wname.top.contact.list configure -editable false
 
 		pack $wname.top.contact.list -side left -expand true -fill both
 		grid $wname.top.contact -row 0 -column 0 -sticky news
 
-		::log::CamLogsByDate $wname $email "1"
+		::log::CamLogsByDate $wname $email 
 
 		button $wname.buttons.play -text "[trans play]" -state $exists \
 		    -command [list ::CAMGUI::Play $wname [file join ${webcam_dir} ${email}.cam]]
 		
-		button $wname.buttons.pause -text "[trans pause]" -command "::CAMGUI::Pause $wname"  -state disabled
-		button $wname.buttons.stop -text "[trans stop]" -command "::CAMGUI::Stop $wname" -state disabled
+		button $wname.buttons.pause -text "[trans pause]" -command [list ::CAMGUI::Pause $wname]  -state disabled
+		button $wname.buttons.stop -text "[trans stop]" -command [list ::CAMGUI::Stop $wname] -state disabled
 
-		button $wname.buttons.save -text "[trans snapshot]" -command "::CAMGUI::saveToImage $wname" -state $exists
-		button $wname.buttons.close -text "[trans close]" -command "destroy $wname"
-		#	button $wname.buttons.stats -text "[trans stats]" -command "::log::cam_stats"
+		button $wname.buttons.save -text "[trans snapshot]" -command [list ::CAMGUI::saveToImage $wname] -state $exists
+		button $wname.buttons.close -text "[trans close]" -command [list destroy $wname]
 
-		button $wname.buttons.clear -text "[trans clearlog]" \
-		    -command "if { !\[winfo exists $wname.top.date.list\] } { \
-				                    set date \".\" \
-				              } else {
-				                    set date \[$wname.top.date.list list get \[$wname.top.date.list curselection\]\]\
-					      }
-                                              if { \[::log::ClearCamLog $email \"\$date\"\] } { 
-				                    destroy $wname
-			         	      }" \
+		button $wname.buttons.clear -text "[trans clearlog]" -command [list ::log::ClearCamLogGUI ${wname}]
 
 		frame $wname.slider -borderwidth 0
 
-		scale $wname.slider.playbackspeed -from 10 -to 500 -resolution 1 -showvalue 1 -label "[trans playbackspeed]" -variable [::config::getVar playbackspeed] -orient horizontal
-		
+		scale $wname.slider.playbackspeed -from 10 -to 1000 -resolution 1 -showvalue 1 -label "[trans playbackspeed]" -variable [::config::getVar playbackspeed] -orient horizontal
+
+		checkbutton $wname.dynamicrate -text "[trans dynamicrate]" -variable [::config::getVar dynamic_rate]
+		frame $wname.dynrate
+		label $wname.dynrate.label -text "[trans currentdynrate] : " 
+		label $wname.dynrate.value -textvariable ::webcam_dynamic_rate
+
 		frame $wname.position -borderwidth 0
 
 		#if { ![info exists ::seek_val($img)] } {
@@ -682,9 +690,9 @@ namespace eval ::log {
 
 		#not using -command to avoid constantly changing while user is dragging it around
 		#interp alias {} imgseek {} ::CAMGUI::Seek $wname [file join ${webcam_dir} ${email}.cam]
-		bind $wname.position.slider <ButtonPress-1> "::CAMGUI::Pause $wname"
-		#bind $wname.position.slider <Button1-ButtonRelease> {imgseek [%W get]}
-		bind $wname.position.slider <ButtonRelease-1> [list ::CAMGUI::Resume $wname [file join ${webcam_dir} ${email}.cam]]
+		bind $wname.position.slider <<Button1-Press>> "::CAMGUI::Pause $wname"
+		#bind $wname.position.slider <<Button1>> {imgseek [%W get]}
+		bind $wname.position.slider <<Button1>> [list ::CAMGUI::Resume $wname [file join ${webcam_dir} ${email}.cam]]
 
 		
 		pack $wname.top -side top -fill x
@@ -699,6 +707,11 @@ namespace eval ::log {
 		pack $wname.buttons.clear -padx 0 -side right
 		pack $wname.buttons.close -padx 0 -side right
 		pack $wname.buttons -side bottom -fill x -pady 3
+		pack $wname.dynrate.label -side left -fill x -pady 3
+		pack $wname.dynrate.value -side left -fill x -pady 3
+		pack $wname.dynrate -side bottom -fill x -pady 3
+		pack $wname.dynrate -side bottom -fill x -pady 3
+		pack $wname.dynamicrate -side bottom -fill x -pady 3
 		pack $wname.slider.playbackspeed -fill x
 		pack $wname.slider -side bottom -fill x -pady 3
 
@@ -714,24 +727,28 @@ namespace eval ::log {
 		set play $wname.buttons.play
 		set pause $wname.buttons.pause
 		set stop $wname.buttons.stop
+		set save $wname.buttons.save
 
-		if {![winfo exists $play] || ![winfo exists $pause] || ![winfo exists $stop] } { return }
+		if {![winfo exists $play] || ![winfo exists $pause] || ![winfo exists $stop] || ![winfo exists $save]} { return }
 
 		switch $button {
 			play {
 				$play configure -state disabled
 				$pause configure -state normal
 				$stop configure -state normal
+				$save configure -state normal
 			}
 			pause {
 				$play configure -state normal
 				$pause configure -state disabled
 				$stop configure -state normal
+				$save configure -state normal
 			}
 			stop {
 				$play configure -state normal
 				$pause configure -state disabled
 				$stop configure -state disabled
+				$save configure -state disabled
 			}
 			default {
 			}
@@ -830,10 +847,10 @@ namespace eval ::log {
 
 	proc wname {email} {
 
-		set wname [split $email "@ ."]
+		set wname [split $email "@ .:"]
 		set wname [join $wname "_"]
 		set wname ".${wname}_hist"
-		return $wname
+		return [string tolower $wname]
 	}
 
 	proc cam_wname {email} {
@@ -841,20 +858,20 @@ namespace eval ::log {
 		set wname [split $email "@ ."]
 		set wname [join $wname "_"]
 		set wname ".${wname}_cam"
-		return $wname
+		return [string tolower $wname]
 	}
 
-	proc LogsByDate {wname email init} {
+	proc LogsByDate {wname email} {
 
 		global log_dir logvar
 
 		#If we store logs by date
 		if { [::config::getKey logsbydate] == 1 } {
 			#If this is the first log we view
-			if {$init == 1} {
+			if {![winfo exists ${wname}.top.date]} {
 				frame $wname.top.date -borderwidth 0
 				combobox::combobox $wname.top.date.list -editable true -highlightthickness 0 -width 22
-			}			
+			}
 			set date_list ""
 			set erdate_list ""
 			$wname.top.date.list list delete 0 end
@@ -899,12 +916,12 @@ namespace eval ::log {
 		}
 	}
 
-	proc CamLogsByDate {wname email init} {
+	proc CamLogsByDate {wname email} {
 
 		global webcam_dir
 
 		# Create the sessions combobox
-		if {$init == 1 } {
+		if {![winfo exists ${wname}.top.sessions]} {
 			frame $wname.top.sessions -borderwidth 0
 			combobox::combobox $wname.top.sessions.list \
 			    -command "::log::JumpToSession $wname" \
@@ -915,7 +932,7 @@ namespace eval ::log {
 		#If we store logs by date
 		if { [::config::getKey logsbydate] == 1 } {
 			#If this is the first log we view
-			if {$init == 1} {
+			if {![winfo exists $wname.top.date]} {
 				frame $wname.top.date -borderwidth 0
 				combobox::combobox $wname.top.date.list -editable true -highlightthickness 0 -width 22
 			}			
@@ -985,57 +1002,6 @@ namespace eval ::log {
 	}
 
 
-	proc ResetSave {w email} {
-
-		global logvar
-
-		set name [::log::wname $email]
-
-		#Redefined the command of the button according to the new contact logging
-		$w.buttons.save configure -command "::log::SaveToFile ${name} ${email} [list ${logvar}]"
-
-	}
-
-	proc ResetCamSave {w email img exists} {
-
-		#Redefined the command of the button according to the new contact logging
-		$w.buttons.save configure -command "::CAMGUI::saveToImage $w" -state $exists
-
-	}
-
-
-	proc ResetDelete {w email} {
-		
-		global logvar
-
-		set name [::log::wname $email]
-
-		#Redefined the command of the button according to the new contact logging
-		$w.buttons.clear configure -command	"if { !\[winfo exists $w.top.date.list\] } { \
-							set date \".\" \
-						} else {
-							set date \[$w.top.date.list list get \[$w.top.date.list curselection\]\]\
-						}
-						if { \[::log::ClearLog $email \"\$date\"\] } { 
-							destroy $w
-						} "
-	}
-
-	proc ResetCamDelete {w email} {
-		
-		set name [::log::wname $email]
-
-		#Redefined the command of the button according to the new contact logging
-		$w.buttons.clear configure -command	"if { !\[winfo exists $w.top.date.list\] } { \
-							set date \".\" \
-						} else {
-							set date \[$w.top.date.list list get \[$w.top.date.list curselection\]\]\
-						}
-						if { \[::log::ClearCamLog $email \"\$date\"\] } { 
-							destroy $w
-						} "
-	}
-
 	proc ChangeLogToDate { w email widget date } {
 
 		global log_dir logvar
@@ -1053,8 +1019,6 @@ namespace eval ::log {
 
 		$w.blueframe.log.txt rodelete 0.0 end
 
-		::log::ResetSave $w $email
-
 
 		if { [file exists [file join ${log_dir} $date ${email}.log]] } {
 			set size "[::amsn::sizeconvert [file size "[file join ${log_dir} $date ${email}.log]"]][trans bytesymbol]"
@@ -1094,14 +1058,12 @@ namespace eval ::log {
 			set exists disabled
 		}
 
-		wm title $w "[trans history2] (${email} - $size)"
+		wm title $w "[trans webcamhistory2] (${email} - $size)"
 
 		set img ${w}_img
 
 		::CAMGUI::Stop $w
 
-		::log::ResetCamSave $w $email $img $exists
-
 		if { ![file exists [file join ${webcam_dir} $date ${email}.cam]] } {
 			set whole_size 0
 		} else {
@@ -1128,7 +1090,7 @@ namespace eval ::log {
 		UpdateSessionList $w $email $date
 	}
 
-	proc ChangeLogWin {w contact widget email} {
+	proc ChangeLogWin {w widget email} {
 
 		global log_dir logvar
 
@@ -1144,10 +1106,7 @@ namespace eval ::log {
 			wm title $w "[trans history2] (${email})"
 		}
 
-		::log::LogsByDate $w $email "0"	
-
-		::log::ResetSave $w $email
-		::log::ResetDelete $w $email
+		::log::LogsByDate $w $email
 
 		ParseLog $w $logvar
 
@@ -1155,7 +1114,7 @@ namespace eval ::log {
 
 	}	
 
-	proc ChangeCamLogWin {w contact {widget ""} {email ""}} {
+	proc ChangeCamLogWin {w {widget ""} {email ""}} {
 
 		global webcam_dir
 
@@ -1173,13 +1132,10 @@ namespace eval ::log {
 			set size "0K[trans bytesymbol]"
 		}
 
-		::log::CamLogsByDate $w $email "0"	
-		::log::ResetCamDelete $w $email
+		::log::CamLogsByDate $w $email 	
 
 		set img ${w}_img
 
-		::log::ResetCamSave $w $email $img $exists
-
 		::CAMGUI::Stop $w
 
 		if { ![file exists [file join ${webcam_dir} ${email}.cam]] } {
@@ -1201,8 +1157,8 @@ namespace eval ::log {
 		$w.buttons.play configure -state $exists \
 		    -command [list ::CAMGUI::Play $w [file join ${webcam_dir} ${email}.cam]]
 		
-		$w.buttons.pause configure -command "::CAMGUI::Pause $w"  -state disabled
-		$w.buttons.stop configure -command "::CAMGUI::Stop $w" -state disabled
+		$w.buttons.pause configure -state disabled
+		$w.buttons.stop configure -state disabled
 
 		
 		wm title $w "[trans webcamhistory2] (${email} - $size)"
@@ -1258,6 +1214,7 @@ namespace eval ::log {
 
 		set loglines [split $logvar "\n"]
 		set result [list]
+		set color grey
 		foreach line $loglines {
 			incr nbline
 			set aidx 0
@@ -1271,11 +1228,22 @@ namespace eval ::log {
 					# The color is indicated by the 6 fingers after it
 					if {[string index $line [expr {$aidx + 3}]] == "C"} {
 						set color [string range $line [expr {$aidx + 4}] [expr {$aidx + 9}]]
-						${wname}.blueframe.log.txt tag configure C_$nbline -foreground "#$color"
-						set color "C_$nbline"
-						incr aidx 10
-						# Else, it is the system with LNOR, LGRA...
+						if { [catch {${wname}.blueframe.log.txt tag configure C_$nbline -foreground "#$color"}]} {
+							# invalid color, maybe the file is corrupted... skip only digits
+							incr aidx 4
+							for {set i 0} { $i < 6 } { incr i } {
+								if { ![string is digit [string range $line $aidx $aidx]] } {
+									break
+								}
+								incr aidx
+							}
+							set color "normal"
+						} else {
+							set color "C_$nbline"
+							incr aidx 10
+						}
 					} else {
+						# Else, it is the system with LNOR, LGRA...
 						if {[string range $line $aidx [expr {$aidx + 6}]] == "\|\"LTIME"  } {
 							#it is a time in [clock seconds]
 							incr aidx 7
@@ -1288,8 +1256,8 @@ namespace eval ::log {
 							set aidx $sidx
 							incr aidx 1
 						} else {
-						set color [string range $line [expr {$aidx + 3}] [expr {$aidx + 5}]]
-						incr aidx 6
+							set color [string range $line [expr {$aidx + 3}] [expr {$aidx + 5}]]
+							incr aidx 6
 						}
 					}
 					set bidx [string first "\|\"L" $line $aidx]
@@ -1347,70 +1315,81 @@ namespace eval ::log {
 		#smile_subst ${wname}.blueframe.log.txt
 	}
 
+	#///////////////////////////////////////////////////////////////////////////////
+	# ClearLogGUI (wname)
+	#
+	# wname : Log window
+
+	proc ClearLogGUI { wname } {
+
+		set email [$wname.top.contact.list list get [$wname.top.contact.list curselection]]
+		
+		if { ![winfo exists $wname.top.date.list] } {
+			set date "."
+		} else {
+			set date [$wname.top.date.list list get [$wname.top.date.list curselection]]
+		}
+        if { [::log::ClearLog $email $date] } { 
+			destroy $wname
+		}
+	}
+
+	#///////////////////////////////////////////////////////////////////////////////
+	# ClearCamLogGUI (wname)
+	#
+	# wname : Log window
+
+	proc ClearCamLogGUI { wname } {
+
+		set email [$wname.top.contact.list list get [$wname.top.contact.list curselection]]
+		
+		if { ![winfo exists $wname.top.date.list] } {
+			set date "."
+		} else {
+			set date [$wname.top.date.list list get [$wname.top.date.list curselection]]
+		}
+        if { [::log::ClearCamLog $email $date] } { 
+			destroy $wname
+		}
+	}
 
 	#///////////////////////////////////////////////////////////////////////////////
 	# SaveToFile (wname email logvar)
-	# File name selection menu and calls ParseToFile
+	# File name selection and calls ParseToTextFile or ParseToHTMLFile, depending on what file type we want to save.
 	#
 	# wname : Log window
 	# logvar : variable containing the whole log file (sure need to setup log file limits)
 
-	proc SaveToFile { wname email logvar } {
-		set wname [string range $wname 1 end]
-		set w .form${wname}
-		toplevel $w
-		wm title $w \"[trans savetofile]\"
-		label $w.msg -justify center -text "Please give a filename"
-		pack $w.msg -side top
+	proc SaveToFile { wname } {
+		global logvar
 
-		frame $w.buttons -class Degt
-		pack $w.buttons -side bottom -fill x -pady 2m
-		button $w.buttons.dismiss -text Cancel -command "destroy $w"
-		button $w.buttons.save -text Save -command "::log::ParseToFile [list ${logvar}] $w.filename.entry; destroy $w"
-		pack $w.buttons.save $w.buttons.dismiss -side left -expand 1
+		set email [$wname.top.contact.list list get [$wname.top.contact.list curselection]]
 
-		frame $w.filename -bd 2 -class Degt
-		entry $w.filename.entry -relief sunken -width 40
-		label $w.filename.label -text "Filename:"
-		pack $w.filename.entry -side right 
-		pack $w.filename.label -side left
-		pack $w.msg $w.filename -side top -fill x
-		focus $w.filename.entry
+		set file [chooseFileDialog "$email.html" "[trans save]" $wname "" save]
 
-		chooseFileDialog $wname "" $w $w.filename.entry save
-	}
-
-
-	#///////////////////////////////////////////////////////////////////////////////
-	# ParseToFile (logvar filepath)
-	# Calls ParseToTextFile or ParseToHTMLFile, depending on what file type we want to save.
-	#
-	# logvar : variable containing the whole log file (sure need to setup log file limits)
-	# filepath : Input box containing file path to save to ??
-
-	proc ParseToFile { logvar filepath } {
-	
-		# Are we writing a HTML file?
-	        set ext [file extension [${filepath} get]]
-		if {$ext == ".htm" ||
-		    $ext == ".html"} {
-			ParseToHTMLFile $logvar $filepath		
-		} else {
-			ParseToTextFile $logvar $filepath
+		if { $file != "" } {
+			# Are we writing a HTML file?
+			set ext [file extension $file]
+			if {$ext == ".htm" ||
+			    $ext == ".html"} {
+				ParseToHTMLFile $logvar $file		
+			} else {
+				ParseToTextFile $logvar $file
+			}
 		}
 	}
 	
 	#///////////////////////////////////////////////////////////////////////////////
-	# ParseToTextFile (logvar filepath)
+	# ParseToTextFile (logvar file)
 	# Decodes the log file and writes to file
 	#
 	# logvar : variable containing the whole log file (sure need to setup log file limits)
-	# filepath : Input box containing file path to save to ??	
-	proc ParseToTextFile { logvar filepath } {
+	# file : filaname
+	proc ParseToTextFile { logvar file } {
 
 		global langenc
 
-		set fileid [open [${filepath} get] a+]
+		set fileid [open $file a+]
 		fconfigure $fileid -encoding utf-8
 		if { $fileid != 0 } {
 			set aidx 0
@@ -1442,7 +1421,7 @@ namespace eval ::log {
 					set aidx $bidx
 				} else {
 					append str [string range $logvar $aidx end]
-					puts -nonewline $fileid str
+					puts -nonewline $fileid $str
 					break
 				}
 				set str ""
@@ -1452,16 +1431,16 @@ namespace eval ::log {
 	}
 	
 	#///////////////////////////////////////////////////////////////////////////////
-	# ParseToHTMLFile (logvar filepath)
+	# ParseToHTMLFile (logvar file)
 	# Decodes the log file and writes to a HTML file
 	#
 	# logvar : variable containing the whole log file (sure need to setup log file limits)
-	# filepath : Input box containing file path to save to ??	
-	proc ParseToHTMLFile { logvar filepath } {
+	# file : filename
+	proc ParseToHTMLFile { logvar file } {
 	
 		global langenc
 
-		set fileid [open [${filepath} get] w+]
+		set fileid [open $file w+]
 		fconfigure $fileid -encoding utf-8
 		if { $fileid != 0 } {
 		
@@ -1487,6 +1466,7 @@ namespace eval ::log {
 
 			set loglines [split $logvar "\n"]
 			set result [list]
+			set color "#efefef"
 			foreach line $loglines {
 				incr nbline
 				set aidx 0
@@ -1603,6 +1583,7 @@ namespace eval ::log {
 			global webcam_dir
 			
 			catch { file delete [file join ${webcam_dir} $date ${email}.cam] }
+			catch { file delete [file join ${webcam_dir} $date ${email}.dat] }
 		}
 		return 1
 	}
@@ -1724,7 +1705,7 @@ namespace eval ::log {
 	#Display/log when we connect if we display/log an event
 	proc eventlogin { } {
 		global eventdisconnected
-		if { $eventdisconnected } {
+		if { ![info exists eventdisconnected] || $eventdisconnected } {
 			set eventdisconnected 0
 			if { [::log::checkeventdisplay] } {
 				.main.eventmenu.list list insert 0 "[timestamp] [trans connectedwith [::config::getKey login]]"
@@ -1743,8 +1724,6 @@ namespace eval ::log {
 	#Display/log when we disconnect if we display/log an event
 	proc eventlogout { } {
 
-		global eventdisconnected
-
 		if { [::log::checkeventdisplay] } {
 			.main.eventmenu.list list insert 0 "[timestamp] : [trans disconnectedfrom [::config::getKey login]]"
 			.main.eventmenu.list select 0
@@ -1815,48 +1794,31 @@ namespace eval ::log {
 	#///////////////////////////////////////////////////////////////////////////////
 	# Sort the contact by the log size
 
-	proc sortalllog { } {
-
+	proc getcontactlogsize { email {month "."} {include_subdirs 1}} {
 		global log_dir
 
-		set contactlist [::abook::getAllContacts]
-		
-		set contactsize [list]
-
-		set months "0 January February March April May June July August September October November December"
-		
-		foreach email $contactlist {
+		set file [file join ${log_dir} ${month} ${email}.log]
 			
+		if { [file exists $file] == 1 } {
+			set size [file size $file]
+		} else {
 			set size 0
-			
-			set file "[file join ${log_dir} ${email}.log]"
-			
-			if { [file exists $file] == 1 } {
-				set size [file size $file]
+		}
+		
+		if {$include_subdirs } {
+			foreach file [glob -nocomplain -types f [file join ${log_dir} * ${email}.log]] {
+				incr size [file size $file]
 			}
+		}
 			
-			if { [::config::getKey logsbydate] == 1 } {
-				foreach date [glob -nocomplain -types f [file join ${log_dir} * ${email}.log]] {
-					set date [getfilename [file dirname $date]]
-					if { [catch { clock scan "1 $date"}] == 0 } {
-						set date [clock scan "1 $date"]
-						set month [clock format $date -format "%m"]
-						if { [string range $month 0 0] == "0" } {
-							set month [string range $month 1 1]
-						}
-						set month "[lindex $months $month]"
-						set year "[clock format $date -format "%Y"]"
-						set date "$month $year"
-						set file [file join ${log_dir} ${date} ${email}.log]
-						if { [file exists $file] } {
-							incr size [file size $file]
-						}
-					}
-				}
-			}
-			
-			set contactsize [lappend contactsize [list $email $size]]
-			
+		return $size
+	}
+
+	proc getlogsizes { {month "."} {include_subdirs 1} } {
+		set contactsize [list]
+		
+		foreach email [::abook::getAllContacts] {
+			lappend contactsize [list $email [::log::getcontactlogsize $email $month $include_subdirs]]
 		}
 		
 		set contactsize [lsort -integer -index 1 -decreasing $contactsize]
@@ -1865,62 +1827,20 @@ namespace eval ::log {
 		
 	}
 
+	proc sortalllog { } {
+		return [::log::getlogsizes]
+		
+	}
+
+	proc sortmonthlog { month } {
+		return [::log::getlogsizes $month 0]
+	}
+
 
 	proc sortthismonthlog { } {
-
-		global log_dir
-
-		set contactlist [::abook::getAllContacts]
-
-		foreach email $contactlist {
-
-			set file [file join ${log_dir} ${email}.log]
-
-			if { [file exists $file] } {
-				set size [file size $file]
-			} else {
-				set size 0
-			}
-
-			set contactsize [lappend contactsize [list $email $size]]
-
-		}
-
-		set contactsize [lsort -integer -index 1 -decreasing $contactsize]
-		
-		return $contactsize
-
+		return [::log::getlogsizes . 0]
 	}
 
-	proc sortmonthlog { month } {
-
-		global log_dir
-		
-		set contactlist [::abook::getAllContacts]
-		
-		set contactsize [list]
-		
-		foreach email $contactlist {
-			
-			set file [file join ${log_dir} ${month} ${email}.log]
-			
-			if { [file exists $file] } {
-				set size [file size $file]
-			} else {
-				set size 0
-			}
-			
-			set contactsize [lappend contactsize [list $email $size]]
-			
-		}
-		
-		set contactsize [lsort -integer -index 1 -decreasing $contactsize]
-		
-		return $contactsize
-		
-	}
-
-
 
 	proc getAllDates { } {
 
@@ -2010,7 +1930,7 @@ namespace eval ::log {
 		pack configure $w.select -side top -fill x -expand false
 
 		frame $w.totalsize
-		label $w.totalsize.txt -text "[trans totalsize] :"
+		label $w.totalsize.txt -text "[trans totalsize]:"
 		pack configure $w.totalsize.txt -side bottom -fill x
 		pack $w.totalsize -side top -fill x -expand false
 
@@ -2034,13 +1954,13 @@ namespace eval ::log {
 			incr id
 			incr totalsize $size
 			set wlabel "label_$id"
-			label $frame.$wlabel -text "$id) $email ([::amsn::sizeconvert $size]b)"
+			label $frame.$wlabel -text "$id) $email ([::amsn::sizeconvert $size][trans bytesymbol])"
 			pack configure $frame.$wlabel -side top
 		}
 		
 		$w.select.list configure -editable false -command "::log::stats_select $id"
 
-		$w.totalsize.txt configure -text "[trans totalsize] : [::amsn::sizeconvert $totalsize]b"
+		$w.totalsize.txt configure -text "[trans totalsize]: [::amsn::sizeconvert $totalsize][trans bytesymbol]"
 
 		#frame $w.button
 		button $w.close -text "[trans close]" -command "destroy $w"
@@ -2085,12 +2005,12 @@ namespace eval ::log {
 			incr id
 			incr totalsize $size
 			set wlabel "label_$id"
-			label $frame.$wlabel -text "$id) $email ([::amsn::sizeconvert $size]b)"
+			label $frame.$wlabel -text "$id) $email ([::amsn::sizeconvert $size][trans bytesymbol])"
 			pack configure $frame.$wlabel -side top
 		}
 		
 		$w.select.list configure -editable false -command "::log::stats_select $id"
-		$w.totalsize.txt configure -text "[trans totalsize] : [::amsn::sizeconvert $totalsize]b"
+		$w.totalsize.txt configure -text "[trans totalsize]: [::amsn::sizeconvert $totalsize][trans bytesymbol]"
 		$w.list.sf yview moveto 0
 		
 		
