@@ -1,11 +1,16 @@
 namespace eval ::winks {
 	
+	variable plugin_path ""
+
 	#----------------------------------------------------------------------------------
 	# DeInit: free resources and unset winks capacity
 	#----------------------------------------------------------------------------------
 	proc DeInit { dir } {
 		# unset tha winks arrays and destroy the menu
-		global HOME winks_list winks_cache
+		global HOME
+		variable winks_list
+		variable winks_cache
+
 		array unset winks_list
 		array unset winks_cache
 		::winks::WinksMenuDestroy 
@@ -22,14 +27,18 @@ namespace eval ::winks {
 		
 		status_log "Loading Winks plugin...\n" green
 
-		global HOME wink_unknown_thumbnail wink_readme_path cabextract_version
-
-		set cabextract_version "undef"
+		global HOME
+		variable wink_unknown_thumbnail
+		variable wink_readme_path
+		variable cabextract_version "undef"
+		variable plugin_path ""
 
 		# this variables helps me to avoid playing two winks at the same time in a chatwindow
-		global winks_f_playing_in
+		variable winks_f_playing_in
 		array unset winks_f_playing_in
 
+		set plugin_path $dir
+
 		# create the directory structure if it doesn't exists
 		create_dir [file join "$HOME" winks]
 		create_dir [file join "$HOME" winks cache]
@@ -48,6 +57,7 @@ namespace eval ::winks {
 		::plugins::RegisterEvent Winks chatwindowbutton AddWinksButton
 		::plugins::RegisterEvent Winks DataCastPacketReceived ReceiveSomething
 		::plugins::RegisterEvent Winks PacketReceived ReceiveSomething
+		::plugins::RegisterEvent Winks OnConnecting OnConnecting
 
 		# load plugin pixmaps
 		::skin::setPixmap unknown_wink unknown_wink.png pixmaps [file join $dir pixmaps]
@@ -106,6 +116,9 @@ namespace eval ::winks {
 			set ::plugins::config(Winks) [array get ::winks::config]
 		} elseif { ! $::winks::config(use_extrac32) } {
 			CheckCabextractVersion
+			if {$cabextract_version == "undef" } {
+				GuessCabextractor "$dir"
+			}
 		}
 		
 		# if there's no flash player configured yet
@@ -119,6 +132,13 @@ namespace eval ::winks {
 		status_log "Winks Loaded OK.\n" green
 	}
 
+	#----------------------------------------------------------------------------------
+	proc OnConnecting { event evpar } {
+		# With the SVN version (current 0.98b) the clientcap is reset to 0 everytime you try to connect
+		# so we need to set our clientcap here now...
+		::MSN::setClientCap winks
+	}
+
 
 	#----------------------------------------------------------------------------------
 	# GuessFlashplayer: finds out what flashplayer command should we use.
@@ -155,6 +175,7 @@ namespace eval ::winks {
 					set ::winks::config(flashplayerargs) ""
 				} else {
 					status_log "No player found." red
+					return 0
 				}
 			} else {
 				if {[file exists [file join $dir "gnash"]]} {
@@ -167,9 +188,11 @@ namespace eval ::winks {
 					set ::winks::config(flashplayerargs) ""
 				} else {
 					status_log "No player found." red
+					return 0
 				}
 			}
 		}
+		return 1
 	}
 	
 	#----------------------------------------------------------------------------------
@@ -185,8 +208,13 @@ namespace eval ::winks {
 			# see if cabextract is present in the system
 			set ::winks::config(cabextractor) "cabextract"
 			if { ! [CheckCabextractVersion] } {
-				# ... see if there's one in the plugin directory
-				if {[file exists [file join $dir "cabextract"]]} {
+				if {[OnMac] && [file exists [file join utils macosx "cabextract"]]} {
+					set ::winks::config(cabextractor) [file join utils macosx "cabextract"]
+					CheckCabextractVersion
+					status_log "cabextract found in utils/macosx path." green
+
+				} elseif {[file exists [file join $dir "cabextract"]]} {
+					# ... see if there's one in the plugin directory
 					set ::winks::config(cabextractor) [file join $dir "cabextract"]
 					CheckCabextractVersion
 					status_log "cabextract found in plugin path." green
@@ -205,7 +233,7 @@ namespace eval ::winks {
 	#                         for correct use of -F arguments
 	#----------------------------------------------------------------------------------
 	proc CheckCabextractVersion { } {
-		global cabextract_version
+		variable cabextract_version
 		if { ! [catch { exec "[FixBars $::winks::config(cabextractor)]" "-v" } ver]  } {
 			if { "$ver" == "cabextract version 0.1" || "$ver" == "cabextract version 0.2" 
 			  || "$ver" == "cabextract version 0.3" || "$ver" == "cabextract version 0.4" 
@@ -226,8 +254,7 @@ namespace eval ::winks {
 	# ShowReadme: Shows a help window with the content of README.txt
 	#----------------------------------------------------------------------------------
 	proc ShowReadme { } {
-	
-		global wink_readme_path
+		variable wink_readme_path
 		
 		if {![file exists $wink_readme_path]} {
 			status_log "Can't open \"$wink_readme_path\"." red
@@ -366,7 +393,8 @@ namespace eval ::winks {
 	proc LoadWinks { } {
 
 		# reset winks array and winks menu
-		global HOME winks_list
+		global HOME
+		variable winks_list
 		array unset winks_list
 		::winks::WinksMenuDestroy 
 		set modified 0
@@ -428,7 +456,8 @@ namespace eval ::winks {
 	proc LoadCache { } {
 		
 		# reset winks cache
-		global HOME winks_cache
+		global HOME
+		variable winks_cache
 		array unset winks_cache
 		set modified 0
 		
@@ -488,7 +517,8 @@ namespace eval ::winks {
 	# SaveWinks: write winks_list in $HOME/winks/index.xml
 	#----------------------------------------------------------------------------------
 	proc SaveWinks { } {
-		global HOME winks_list
+		global HOME
+		variable winks_list
 	        set fileId [open [file join "$HOME" winks index.xml] "w"]
 		foreach sha1d [array names winks_list] {
 			array set wink $winks_list($sha1d)
@@ -503,7 +533,8 @@ namespace eval ::winks {
 	# SaveCache: write winks_list in $HOME/winks/index.xml
 	#----------------------------------------------------------------------------------
 	proc SaveCache { } {
-		global HOME winks_cache
+		global HOME
+		variable winks_cache
 	        set fileId [open [file join "$HOME" winks cache index.xml] "w"]
 		foreach sha1d [array names winks_cache] {
 			array set wink $winks_cache($sha1d)
@@ -527,20 +558,21 @@ namespace eval ::winks {
 		set buttonbar $newvar(bottom)		
 
 		set winksbut $buttonbar.winksbut
-		label $winksbut -image [::skin::loadPixmap butwinks] -relief flat -padx 0 \
+                set window $newvar(window_name)
+                set chatid [::ChatWindow::Name $window]
+                set winks_f_playing_in($chatid) 0
+
+		button $winksbut -image [::skin::loadPixmap butwinks] -relief flat -padx 0 \
 			-background [::skin::getKey buttonbarbg] -highlightthickness 0 -borderwidth 0 \
-			-highlightbackground [::skin::getKey buttonbarbg] -activebackground [::skin::getKey buttonbarbg]
+			-highlightbackground [::skin::getKey buttonbarbg] -activebackground [::skin::getKey buttonbarbg] \
+			-command "::winks::WinksMenu $window \[winfo pointerx $window\] \[winfo pointery $window\] [::ChatWindow::GetInputText $window] \"\" 0"
 		pack $winksbut -side left -padx 0 -pady 0
 
-		set window $newvar(window_name)
-		set chatid [::ChatWindow::Name $window]
-		set winks_f_playing_in($chatid) 0
-		
 		bind $winksbut <<Button2>> "::winks::WinksMenuDestroy"
 		bind $winksbut  <<Button3>> "::winks::AddWinkFromMCO $window"
 		bind $winksbut  <Enter> "$winksbut configure -image [::skin::loadPixmap butwinks_hover]"
 		bind $winksbut  <Leave> "$winksbut configure -image [::skin::loadPixmap butwinks]"	
-		bind $winksbut  <<Button1>> "::winks::WinksMenu $window \[winfo pointerx $window\] \[winfo pointery $window\] [::ChatWindow::GetInputText $window] \"\" 0"
+		#bind $winksbut  <<Button1>> "::winks::WinksMenu $window \[winfo pointerx $window\] \[winfo pointery $window\] [::ChatWindow::GetInputText $window] \"\" 0"
 	}
 	
 	
@@ -556,7 +588,7 @@ namespace eval ::winks {
 	#----------------------------------------------------------------------------------
 	proc winksCreateMenu { } {
 	
-		global winks_list
+		variable winks_list
 
 		set w .winksSelector
 
@@ -635,7 +667,7 @@ namespace eval ::winks {
 	# winksCreateInMenu: add a wink button to the winks menu
 	#----------------------------------------------------------------------------------
 	proc winksCreateInMenu {w cols rows smiw smih wink_num wink_pos name symbol image} {
-		global winks_list
+		variable winks_list
 		catch {
 			set resized [image create photo]
 			$resized copy $image
@@ -667,7 +699,7 @@ namespace eval ::winks {
 
 		set chatid [::ChatWindow::Name $window_name]
 
-		global winks_list
+		variable winks_list
 		
 		set w .winksSelector
 		
@@ -701,14 +733,14 @@ namespace eval ::winks {
 				#the winks menu for first time, the second click can launch
 				#this procedure without all winks having been created
 				catch { 
-					bind $w.c.$temp <Button1-ButtonRelease> "wm state $w withdrawn; ::MSN::ChatQueue $chatid \[list ::winks::SendWink $window_name $wink(sha1d)]"
-					bind $w.c.$temp <Button2-ButtonRelease> "::winks::PlayWink \"$wink(swf)\""
-					bind $w.c.$temp <Button3-ButtonRelease> "wm state $w withdrawn; ::winks::EditWinkDialog $chatid \"$wink(sha1d)\""
+					bind $w.c.$temp <<Button1>> "wm state $w withdrawn; ::MSN::ChatQueue $chatid \[list ::winks::SendWink $window_name $wink(sha1d)]"
+					bind $w.c.$temp <<Button2>> "::winks::PlayWink \"$wink(swf)\""
+					bind $w.c.$temp <<Button3>> "wm state $w withdrawn; ::winks::EditWinkDialog $chatid \"$wink(sha1d)\""
 				}
 				incr temp
 			}
 		}
-		bind $w.c.new_but <Button1-ButtonRelease> "wm state $w withdrawn; ::winks::AddWinkFromMCO $window_name"
+		bind $w.c.new_but <<Button1>> "wm state $w withdrawn; ::winks::AddWinkFromMCO $window_name"
 		moveinscreen $w 5
 		event generate $w <Enter>
 	
@@ -719,7 +751,7 @@ namespace eval ::winks {
 	#----------------------------------------------------------------------------------
 	proc EditWinkDialog { chatid sha1d } {
 			set w .editWink
-			global winks_list
+			variable winks_list
 			array set wink $winks_list($sha1d)
 			if { [winfo exists .editWink]} {destroy .editWink}
 			toplevel $w
@@ -762,7 +794,7 @@ namespace eval ::winks {
 	# RenameWink: called from EditWinkDialog, changes a wink name
 	#----------------------------------------------------------------------------------
 	proc RenameWink { chatid sha1d } {
-		global winks_list
+		variable winks_list
 		set wname [.editWink.fn.name get]
 		array set wink $winks_list($sha1d)
 		# if the name is empty, find a suitable one
@@ -783,7 +815,9 @@ namespace eval ::winks {
 	# DeleteWink: called from EditWinkDialog, delete a wink from winks_list
 	#----------------------------------------------------------------------------------
 	proc DeleteWink { chatid sha1d } {
-		global HOME winks_cache winks_list
+		global HOME
+		variable winks_cache
+		variable winks_list
 		set wname [.editWink.fn.name get]
 		array set wink $winks_list($sha1d)
 
@@ -877,7 +911,7 @@ namespace eval ::winks {
 			set wink(sha1d) [::base64::encode [binary format H* [::sha1::sha1 $data]]]
 			
 			# see if the wink is already in the winks menu
-			global winks_list
+			variable winks_list
 			foreach wsha1d [array names winks_list] {
 				if { "$wink(sha1d)" == "$wsha1d" } {
 					status_log "The wink is already in winks menu!" red
@@ -925,7 +959,7 @@ namespace eval ::winks {
 			}
 			
 			# copy information to cache
-			global winks_cache
+			variable winks_cache
 			set winks_cache($wink(sha1d)) [array get wink]
 			SaveCache
 			
@@ -981,22 +1015,16 @@ namespace eval ::winks {
 	# AddWinkDialoOK: change the wink name and moves it to winks menu 
 	#----------------------------------------------------------------------------------
 	proc AddWinkDialoOK { chatid sha1d } {
-status_log "BOOGA"
-		global winks_cache
-status_log "BOOGA"
+		variable winks_cache
 		array set wink $winks_cache($sha1d)
-status_log "BOOGA"
 
 		set wink_name [.addNewWink.fn.name get]
 		if { "$wink_name" != "" } {
 			set wink(name) "$wink_name"
 			set winks_cache($sha1d) [array get wink]
 		}
-status_log "BOOGA"
 		destroy .addNewWink
-status_log "BOOGA"
 		::winks::AddWinkFromCache $chatid $sha1d
-status_log "BOOGA"
 	}
 
 	#----------------------------------------------------------------------------------
@@ -1004,7 +1032,8 @@ status_log "BOOGA"
 	#----------------------------------------------------------------------------------
 	proc PlayWinkFromSHA1D { chatid sha1d } {
 		
-		global winks_list winks_cache
+		variable winks_list
+		variable winks_cache
 		
 		# see if the wink is in our winks menu to play it
 		foreach wsha1d [array names winks_list] {
@@ -1056,10 +1085,17 @@ status_log "BOOGA"
 		# old play wink code
 		status_log "Playing wink $wfile\n" green
 		set command [concat "exec" "\"[FixBars $::winks::config(flashplayer)]\"" [split "$::winks::config(flashplayerargs)" " "] "\"$wfile\"" "&"]
-		catch { eval $command } errs
-		if { "$errs" != "" } {
-			status_log "\nEval: $command\n" red
-			status_log "\nOutput: $errs\n" red
+		if {[catch { eval $command } errs] } {
+			variable plugin_path
+			if {[GuessFlashplayer $plugin_path] } {
+				if {[catch { eval $command } errs] } {
+					status_log "\nEval2: $command\n" red
+					status_log "\nOutput: $errs\n" red
+				}
+			} else {
+				status_log "\nEval: $command\n" red
+				status_log "\nOutput: $errs\n" red
+			}
 		}
 		
 		return
@@ -1070,7 +1106,12 @@ status_log "BOOGA"
 	#----------------------------------------------------------------------------------
 	proc PlayWinkInChatWindow { wfile chatid wkx wky } {
 
-		global winks_f_playing_in
+		variable winks_f_playing_in
+
+		if {![OnLinux] } {
+			return [PlayWink $wfile]
+		}
+
 		if { ![info exists winks_f_playing_in($chatid)] } {
 			set winks_f_playing_in($chatid) 0
 		}
@@ -1120,15 +1161,41 @@ status_log "BOOGA"
 		
         	# play the animation
 		status_log "Playing wink $wfile\n" green
-		set io [open "|[FixBars $::winks::config(flashplayer)] -1 -x [winfo id $outframe.wink_canvas] -j $sx -k $sy $wfile" r]
+		if {[catch { 	set io [open "|[FixBars $::winks::config(flashplayer)] -1 -x [winfo id $outframe.wink_canvas] -j $sx -k $sy $wfile" r] } ] } {
+			variable plugin_path
+			if {[GuessFlashplayer $plugin_path] } {
+				if {[catch { 	set io [open "|[FixBars $::winks::config(flashplayer)] -1 -x [winfo id $outframe.wink_canvas] -j $sx -k $sy $wfile" r] } ] } {
+					status_log "Couldn't launch the player"
+					# delete the temp widget and show the text again
+					destroy $outframe.wink_canvas
+					eval pack $outtext $wink_text_pack
+		
+					# say that we finished playing
+					set winks_f_playing_in($chatid) 0
+					PlayWink $wfile
+				}
+			} else {
+				status_log "Couldn't find proper player"
+				# delete the temp widget and show the text again
+				destroy $outframe.wink_canvas
+				eval pack $outtext $wink_text_pack
+		
+				# say that we finished playing
+				set winks_f_playing_in($chatid) 0
+				PlayWink $wfile
+			}
+		}
 		fconfigure $io -blocking 0
 		gets $io line
-
+		status_log "Read $line"
+			
 		while { ![eof $io] } { 
-			gets $io line
+			after 1000
 			update
+			gets $io line
+			status_log "Read $line"
 		}
-
+	
 		# delete the temp widget and show the text again
 		destroy $outframe.wink_canvas
 		eval pack $outtext $wink_text_pack
@@ -1154,7 +1221,7 @@ status_log "BOOGA"
 		} else {
 
 			# check cabextract version for correct use of -F arguments
-			global cabextract_version
+			variable cabextract_version
 			if { "$cabextract_version" == "undef" } {
 				CheckCabextractVersion
 			}	
@@ -1201,7 +1268,9 @@ status_log "BOOGA"
 	#----------------------------------------------------------------------------------
 	proc ReceiveSomething { event evpar } {
 
-		global winks_list winks_cache HOME
+		global HOME
+		variable winks_list
+		variable winks_cache
 
 		upvar 2 $evpar args
 		upvar 2 $args(chatid) chatid
@@ -1293,7 +1362,8 @@ status_log "BOOGA"
 	#----------------------------------------------------------------------------------
 	proc ReceivedWink { event evpar } {
 		
-		global HOME winks_cache
+		global HOME
+		variable winks_cache
 
 		upvar 2 $evpar args
 		upvar 2 $args(chatid) chatid
@@ -1391,10 +1461,10 @@ status_log "BOOGA"
 	proc WinWriteThumbnail { chatid sha1d { cache 0 } } {
 		
 		if { "$cache" == 1 } {
-			global winks_cache
+			variable winks_cache
 			array set wink $winks_cache($sha1d)
 		} else {
-			global winks_list
+			variable winks_list
 			array set wink $winks_list($sha1d)
 		}
 
@@ -1409,7 +1479,7 @@ status_log "BOOGA"
 			set thumbnail [image create photo $twTag -file $wink(img) -format cximage]
 		} else {
 			# if we don't have the thumbnail yet, we'll show the "please wait..." one
-			global wink_unknown_thumbnail
+			variable wink_unknown_thumbnail
 			set thumbnail [image create photo $twTag -file "$wink_unknown_thumbnail" -format cximage]
 		}
 
@@ -1458,7 +1528,7 @@ status_log "BOOGA"
 	#----------------------------------------------------------------------------------
 	proc SendWink { window_name sha1d } {
 		
-		global winks_list
+		variable winks_list
 		array set wink $winks_list($sha1d)
 
 		set chatid [::ChatWindow::Name $window_name]
@@ -1466,7 +1536,7 @@ status_log "BOOGA"
 		# avoid multi-converstation bug
 		if { "[::abook::getContactData $chatid clientid]" != "" } {
 			# check if the other client support winks
-			if { [expr {[::abook::getContactData $chatid clientid] & 0x008000}] == 0 } {
+			if { [::MSN::hasCapability [::abook::getContactData $chatid clientid] winks] == 0 } {
 				amsn::WinWrite $chatid "\n[::abook::getDisplayNick $chatid] [trans winks_yourcontact_amsn_client_doesnt_supports_winks] " green 
 				return
 			}
@@ -1529,7 +1599,9 @@ status_log "BOOGA"
 	#----------------------------------------------------------------------------------
 	proc AddWinkFromCache { chatid sha1d } { 
 		
-		global HOME winks_cache winks_list
+		global HOME
+		variable winks_cache
+		variable winks_list
 		
 		# see if the wink is in our wink menu
 		foreach wsha1d [array names winks_list] {
