@@ -1,5 +1,5 @@
 
-::Version::setSubversionId {$Id: roaming.tcl 11891 2010-01-11 20:18:34Z vivia $}
+::Version::setSubversionId {$Id: roaming.tcl 12199 2010-10-18 21:51:03Z baaazen $}
 
 snit::type ContentRoaming {
 	# GetProfile, UpdateProfile,
@@ -118,6 +118,9 @@ snit::type ContentRoaming {
 						set dp_resourceid [GetXmlEntry $result "GetProfileResult:ExpressionProfile:Photo:ResourceID"]
 						::abook::setPersonal dp_resourceid $dp_resourceid
 						set dp_filename [GetXmlEntry $result "GetProfileResult:ExpressionProfile:Photo:Name"]
+						if {$dp_filename == ""} {
+							set dp_filename $dp_resourceid
+						}
 						::abook::setPersonal dp_filename $dp_filename
 						set dp_url [GetXmlEntry $result "GetProfileResult:ExpressionProfile:StaticUserTilePublicURL"]
 						::abook::setPersonal dp_url $dp_url
