@@ -1,20 +1,21 @@
 
-::Version::setSubversionId {$Id: trayicon.tcl 8970 2007-08-23 05:29:13Z kakaroto $}
+::Version::setSubversionId {$Id: trayicon.tcl 12216 2010-11-13 21:44:28Z kakaroto $}
 
 if { $initialize_amsn == 1 } {
-    global statusicon mailicon systemtray_exist iconmenu ishidden defaultbackground
+    global statusicon mailicon systemtray_exist iconmenu ishidden defaultbackground use_tktray
 
     set statusicon 0
     set mailicon 0
     set systemtray_exist 0
     set iconmenu 0
     set ishidden 0
+    set use_tktray 0
 }
 
 proc iconify_proc {} {
 	global statusicon systemtray_exist
 	
-	if { [OnWin] } {
+	if { [WinDock] } {
 		taskbar_icon_handler WM_LBUTTONDBLCLK 0 0
 	} else {
 		if { [winfo exists .bossmode] } {
@@ -23,7 +24,11 @@ proc iconify_proc {} {
 			focus -force .bossmode
 			return
 		}
-		if { [focus] == "."} {
+		set focus [focus]
+		if {[winfo exists $focus] } {
+			set focus [winfo toplevel $focus]
+		}
+		if {  $focus == "." && [wm state .] != "iconic" && [wm state .] != "withdrawn"} {
 			wm iconify .
 			wm state . withdrawn
 		} else {
@@ -33,7 +38,6 @@ proc iconify_proc {} {
 			focus -force .
 		}
 	}
-	#bind $statusicon <Button-1> deiconify_proc
 }
 
 
@@ -48,7 +52,7 @@ proc isWinicoLoaded {} {
 
 # Load the needed library (platform dependent) and create the context-menu
 proc trayicon_init {} {
-	global systemtray_exist password iconmenu wintrayicon statusicon
+	global systemtray_exist iconmenu wintrayicon statusicon use_tktray
 
 	if { [WinDock] } {
 		#added to stop creation of more than 1 icon
@@ -83,15 +87,37 @@ proc trayicon_init {} {
 		destroy .trayiconwin.immain
 		set iconmenu .trayiconwin.immain
 
-	} else {
-		if { [catch {package require libtray} res] } {
+	} elseif {[UnixDock]} {
+               if { [catch {package require tktray} res] } {
+		       if { [catch {package require libtray} res] } {
+			       set systemtray_exist 0
+			       status_log "[trans traynotcompiled] : $res"
+			       close_dock
+			       return
+		       } else {
+			       set use_tktray 0
+		       }
+	       } else {
+		       set use_tktray 1
+	       }
+
+		if {$use_tktray} {
+			set systemtray_exist 1
+		} else {
+			set systemtray_exist [systemtray_exist]; #a system tray exist?
+		}
+
+		destroy .immain
+		set iconmenu .immain
+	} elseif {[MacDock]} {
+		if { [catch {package require statusicon} res] } {
 			set systemtray_exist 0
-			status_log "[trans traynotcompiled] : $res"
+			status_log "Mac statusicon package not found : $res"
 			close_dock
 			return
 		}
 
-		set systemtray_exist [systemtray_exist]; #a system tray exist?
+		set systemtray_exist 1
 
 		destroy .immain
 		set iconmenu .immain
@@ -122,6 +148,7 @@ proc trayicon_init {} {
        $iconmenu add separator
  
        $iconmenu add command -label "[trans changenick]..." -command cmsn_change_name -state disabled
+       $iconmenu add command -label "[trans changepsm]..." -command cmsn_change_name -state disabled
        $iconmenu add checkbutton -label "[trans sound]" -onvalue 1 -offvalue 0 -variable [::config::getVar sound]
        $iconmenu add checkbutton -label "[trans shownotify]" -onvalue 1 -offvalue 0 -variable [::config::getVar shownotify]
  #     $iconmenu add command -label "[trans preferences]..." -command Preferences
@@ -161,6 +188,8 @@ proc taskbar_icon_handler { msg x y } {
 		#tk_popup $iconmenu $x $y
 
 		#workaround for bug with the popup not unposting
+		# TODO: verify if this bug is only for old tk versions and/or if it happens on older windows versions
+		# Tested with Win7 and Tk 8.5.7 and it seems just fine without the workaround!
 		wm state .trayiconwin normal
 		wm geometry .trayiconwin "+0+[expr {2 * [winfo screenheight .]}]"
 		focus -force .trayiconwin
@@ -196,62 +225,122 @@ proc taskbar_icon_handler { msg x y } {
 }
 
 proc trayicon_callback {imgSrc imgDst width height} {
-	$imgDst copy $imgSrc -compositingrule set
-	if { [image width $imgSrc] > $width || [image height $imgSrc] > $height} {
-		::picture::ResizeWithRatio $imgDst $width $height
+       $imgDst copy $imgSrc -compositingrule set
+       if { [image width $imgSrc] > $width || [image height $imgSrc] > $height} {
+               ::picture::ResizeWithRatio $imgDst $width $height
+       }
+}
+
+proc trayicon_resize {w} {
+	catch {
+		set width [expr {[lindex [$w bbox] 2] - [lindex [$w bbox] 0] + 1}]
+		set height [expr {[lindex [$w bbox] 2] - [lindex [$w bbox] 0] + 1}]
+			
+		set img [$w cget -image]
+		if { [image width $img] != $width || [image height $img] != $height} {
+			::picture::ResizeWithRatio $img $width $height
+		}
+	}
+}
+
+proc statusicon_callback { action } {
+	global statusicon_callback_afterid
+
+	if { [info exists statusicon_callback_afterid] } {
+		catch { after cancel $statusicon_callback_afterid }
+	}
+	set statusicon_callback_afterid [after 250 [list statusicon_callback_delayed $action [winfo pointerx .] [winfo pointery .]]]
+}
+
+proc statusicon_callback_delayed { action x y} {
+	global iconmenu
+
+	if { $action == "ACTION" } {
+		tk_popup $iconmenu $x $y
+	}
+	if { $action == "DOUBLE_ACTION" } {
+		iconify_proc
 	}
 }
 
 proc statusicon_proc {status} {
-	global systemtray_exist statusicon list_states iconmenu wintrayicon defaultbackground
+	global systemtray_exist statusicon list_states iconmenu wintrayicon defaultbackground use_tktray
 	set cmdline ""
 
-	if { [::config::getKey use_tray] == 0 } { return }
+	if { [::config::getKey use_tray] == 0 && $status != "REMOVE"} { return }
 
 	if { ![WinDock] } {
 
-		if { $systemtray_exist == 1 && $statusicon == 0 && [UnixDock]} {
+		if { $systemtray_exist == 1 && $statusicon == 0 && $status != "REMOVE" && [UnixDock]} {
 			set pixmap "[::skin::GetSkinFile pixmaps doffline.png]"
 			image create photo statustrayicon -file $pixmap
-			image create photo statustrayiconres
-			#add the icon
-			set statusicon [newti .si -tooltip offline -pixmap statustrayiconres -command "::trayicon_callback statustrayicon statustrayiconres"]
+			set statusicon .si
 
-			bind .si <Button1-ButtonRelease> iconify_proc
-			bind .si <Button3-ButtonRelease> "tk_popup $iconmenu %X %Y"
+			#add the icon
+			if {$use_tktray} {
+				tktray::icon .si -class "amsn-tray" -image statustrayicon
+				after 1000 { trayicon_resize .si}
+				bind $statusicon <<IconConfigure>> {trayicon_resize .si}
+			} else {
+				image create photo statustrayiconres
+				set statusicon [newti .si -pixmap statustrayiconres -command "::trayicon_callback statustrayicon statustrayiconres"]
+			}
+
+			bind $statusicon <<Button1>> iconify_proc
+			bind $statusicon <<Button3>> "tk_popup $iconmenu %X %Y"
+		}
+		if { $systemtray_exist == 1 && $statusicon == 0 && $status != "REMOVE" && [MacDock]} {
+			set pixmap "[::skin::GetSkinFile pixmaps doffline.png]"
+			#add the icon
+			set statusicon [::statusicon::create statusicon_callback]
+			::statusicon::setTooltip $statusicon "[trans offline]"
+			::statusicon::setImage $statusicon $pixmap
+		}
+	} else {
+		if { $systemtray_exist == 1 && $statusicon == 0 && $status != "REMOVE" } {
+			trayicon_init
 		}
 	}
 
 	set my_name [::abook::getPersonal MFN]
    	
 	if { $systemtray_exist == 1 && $statusicon != 0 && $status == "REMOVE" } {
-		if { [OnWin] } {
+		if { [WinDock] } {
 			winico taskbar delete $wintrayicon
 		} else {
 			remove_icon $statusicon
-			destroy trayicon
+			if {[UnixDock] } {
+				catch {
+					image delete statustrayicon
+					if {!$use_tktray} {
+						image delete statustrayiconres
+					}
+				}
+			}
 		}
 		set statusicon 0
-	} elseif {$systemtray_exist == 1 && $statusicon != 0 && ( [UnixDock] || [WinDock] ) && $status != "REMOVE"} {
+	} elseif {$systemtray_exist == 1 && $statusicon != 0 && $status != "REMOVE"} {
 		if { $status != "" } {
 			if { $status == "FLN" } {
 				# Send message 
 				$iconmenu entryconfigure 2 -state disabled
 				#Status submenu
 				$iconmenu entryconfigure 4 -state disabled
-				#Change nick
+				#Change nick/psm
 				$iconmenu entryconfigure 6 -state disabled
+				$iconmenu entryconfigure 7 -state disabled
 
 				#Login/Logout
-				$iconmenu entryconfigure 12 -label "[trans login]" -command "::MSN::connect" -state normal
+				$iconmenu entryconfigure 13 -label "[trans login]" -command "::MSN::connect" -state normal
 
 			} else {
 
 				$iconmenu entryconfigure 2 -state normal
 				$iconmenu entryconfigure 4 -state normal
 				$iconmenu entryconfigure 6 -state normal
+				$iconmenu entryconfigure 7 -state normal
 
-				$iconmenu entryconfigure 12 -label "[trans logout]" -command "preLogout \"::MSN::logout\""
+				$iconmenu entryconfigure 13 -label "[trans logout]" -command "preLogout \"::MSN::logout\""
 
 				
 			}
@@ -346,16 +435,22 @@ proc statusicon_proc {status} {
 					winico taskbar delete $wintrayicon
 					set wintrayicon $trayicon
 					winico taskbar add $wintrayicon -text $tooltip -callback "taskbar_icon_handler %m %x %y"
+					set statusicon 1
 				}
 
-			} else {
+			} elseif {[UnixDock] } {
 				if { $pixmap != "null"} {
-					configureti $statusicon -tooltip $tooltip
 					image create photo statustrayicon -file $pixmap
-					image create photo statustrayiconres
-					#if { [image width statustrayicon] > [winfo width $icon] || [image height statustrayicon] > [winfo height $icon]} {
-					#	::picture::ResizeWithRatio statustrayicon [winfo width $icon] [winfo height $icon]
-					#}
+					if {!$use_tktray} {
+						image create photo statustrayiconres
+					}
+				}
+			} elseif {[MacDock] } {
+				if { $pixmap == "null"} {
+					::statusicon::setVisible $statusicon 0
+				} else {
+					::statusicon::setImage $statusicon $pixmap
+					::statusicon::setTooltip $statusicon $tooltip
 				}
 			}
 
@@ -363,32 +458,155 @@ proc statusicon_proc {status} {
 	}
 }
 
-proc taskbar_mail_icon_handler { msg x y } {
-	global password
+proc statusicon_blink_proc {status} {
+    global systemtray_exist statusicon iconmenu wintrayicon use_tktray
 
-	if { [winfo exists .bossmode] } {
-		if { $msg=="WM_LBUTTONDBLCLK" } {
-			wm state .bossmode normal
-			focus -force .bossmode
-		}
-		return
+    if { [::config::getKey use_tray] == 0 || $systemtray_exist == 0 || $statusicon == 0 } { return }
+
+    set my_name [::abook::getPersonal MFN]
+    
+    if { $status != "" } {
+        switch $status {
+          "FLN" {
+            set pixmap "[::skin::GetSkinFile pixmaps dofflineb.png]"
+            set tooltip "[trans offline]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons offlineb.ico]]
+            }
+          }
+        
+          "NLN" {
+            set pixmap "[::skin::GetSkinFile pixmaps donlineb.png]"
+            set tooltip "$my_name ([::config::getKey login]): [trans online]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons onlineb.ico]]
+            }
+          }
+          
+          "IDL" {
+            set pixmap "[::skin::GetSkinFile pixmaps dinactiveb.png]"
+            set tooltip "$my_name ([::config::getKey login]): [trans noactivity]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons inactiveb.ico]]
+            }
+          }
+          "BSY" {
+            set pixmap "[::skin::GetSkinFile pixmaps dbusyb.png]"
+            set tooltip "$my_name ([::config::getKey login]): [trans busy]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons busyb.ico]]
+            }
+          }
+          "BRB" {
+            set pixmap "[::skin::GetSkinFile pixmaps dbrbb.png]"
+            set tooltip "$my_name ([::config::getKey login]): [trans rightback]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons brbb.ico]]
+            }
+          }
+          "AWY" {
+            set pixmap "[::skin::GetSkinFile pixmaps dawayb.png]"
+            set tooltip "$my_name ([::config::getKey login]): [trans away]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons awayb.ico]]
+            }
+          }
+          "PHN" {
+            set pixmap "[::skin::GetSkinFile pixmaps dphoneb.png]"
+            set tooltip "$my_name ([::config::getKey login]): [trans onphone]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons phoneb.ico]]
+            }
+          }
+          "LUN" {
+            set pixmap "[::skin::GetSkinFile pixmaps dlunchb.png]"
+            set tooltip "$my_name ([::config::getKey login]): [trans gonelunch]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons lunchb.ico]]
+            }
+          }
+          "HDN" {
+            set pixmap "[::skin::GetSkinFile pixmaps dhiddenb.png]"
+            set tooltip "$my_name ([::config::getKey login]): [trans appearoff]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons hiddenb.ico]]
+            }
+          }
+          "BOSS" {   
+            #for bossmode
+            set pixmap "[::skin::GetSkinFile pixmaps dboss.png]"
+            set tooltip "[trans pass]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons bossmode.ico]]
+            }
+          }
+          default {
+            set pixmap "null"
+            set tooltip ""
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons msn.ico]]
+            }
+          }
+        }
+
+	if {[::skin::getKey trayblink_by_status] == 0} {
+            set pixmap "[::skin::GetSkinFile pixmaps trayblink.png]"
+            if { [WinDock] } {
+                set trayicon [winico create [::skin::GetSkinFile winicons trayblink.ico]]
+            }
 	}
 
+        $iconmenu entryconfigure 0 -label "[::config::getKey login]"
+        if { [WinDock] } {
+            if { ![winfo exists .bossmode] || $status == "BOSS" } {
+                #skip if in bossmode and not setting to BOSS
+                winico taskbar delete $wintrayicon
+                set wintrayicon $trayicon
+                winico taskbar add $wintrayicon -text $tooltip -callback "taskbar_icon_handler %m %x %y"
+                set statusicon 1
+            }
+
+        } elseif {[UnixDock] } {
+            if { $pixmap != "null"} {
+                image create photo statustrayicon -file $pixmap
+		if {!$use_tktray} {
+			image create photo statustrayiconres
+		}
+            }
+        } elseif {[MacDock] } {
+            if { $pixmap == "null"} {
+                ::statusicon::setVisible $statusicon 0
+            } else {
+                ::statusicon::setImage $statusicon $pixmap
+                ::statusicon::setTooltip $statusicon $tooltip
+            }
+        }
+
+    }
+}
+
+proc taskbar_mail_icon_handler { msg x y } {
 	if { $msg=="WM_LBUTTONUP" } {
 		::hotmail::hotmail_login 
 	}
 }
 
+proc mailicon_callback { action } {
+	if { $action == "DOUBLE_ACTION" } {
+		::hotmail::hotmail_login
+	}   
+}
+
 proc mailicon_proc {num} {
 	# Workaround for bug in the traydock-plugin - statusicon added - BEGIN
-	global systemtray_exist mailicon statusicon password winmailicon mailtrayicon defaultbackground
+	global systemtray_exist mailicon statusicon winmailicon defaultbackground use_tktray
 	# Workaround for bug in the traydock-plugin - statusicon added - END
 
 	if { [::config::getKey showmailicon] == 0 } {
 		return
 	}
 
-	if {$systemtray_exist == 1 && $mailicon == 0 && ([UnixDock] || [WinDock])  && $num >0} {
+	if {$systemtray_exist == 1 && $mailicon == 0 && $num >0} {
 		set pixmap "[::skin::GetSkinFile pixmaps unread_tray.gif]"
 		if { $num == 1 } {
 			set msg [trans onenewmail]
@@ -402,27 +620,47 @@ proc mailicon_proc {num} {
 			set winmailicon [winico create [::skin::GetSkinFile winicons unread.ico]]
 			winico taskbar add $winmailicon -text $msg -callback "taskbar_mail_icon_handler %m %x %y"
 			set mailicon 1
-		} else {
+		} elseif {[UnixDock] } {
 			image create photo mailtrayicon -file $pixmap
-			image create photo mailtrayiconres
-			set mailicon [newti .mi -tooltip offline -pixmap mailtrayiconres -command "::trayicon_callback mailtrayicon mailtrayiconres"]
+			if {$use_tktray} {
+				catch {destroy .mi}
+				set mailicon .mi
+				tktray::icon .mi -image mailtrayicon
+				after 1000 { trayicon_resize .mi}
+
+				bind .mi <<IconConfigure>> {trayicon_resize .mi}
+			} else {
+				image create photo mailtrayiconres
+				set mailicon [newti .mi -pixmap mailtrayiconres -command "::trayicon_callback mailtrayicon mailtrayiconres"]
+			} 
 
 			bind .mi <Button-1> "::hotmail::hotmail_login"
 			bind .mi <Enter> [list balloon_enter %W %X %Y $msg]
 			bind .mi <Motion> [list balloon_motion %W %X %Y $msg]
 			bind .mi <Leave> "+set Bulle(first) 0; kill_balloon"
+		} elseif {[MacDock] } {
+			#add the icon
+			set mailicon [::statusicon::create mailicon_callback]
+			::statusicon::setTooltip $mailicon "[trans onenewmail]"
+			::statusicon::setImage $mailicon $pixmap
 		}
 
 	} elseif {$systemtray_exist == 1 && $mailicon != 0 && $num == 0} {
-		if { [OnWin] } {
+		if { [WinDock] } {
 			winico taskbar delete $winmailicon
-			set mailicon 0
 		} else {
 			remove_icon $mailicon
-			destroy mailtrayicon
-			set mailicon 0
+			if {[UnixDock] } {
+				catch {
+					image delete mailtrayicon
+					if {!$use_tktray} {
+						image delete mailtrayiconres
+					}
+				}
+			}
 		}
-	} elseif {$systemtray_exist == 1 && $mailicon != 0 && ([UnixDock] || [WinDock])  && $num > 0} {
+		set mailicon 0
+	} elseif {$systemtray_exist == 1 && $mailicon != 0 && $num > 0} {
 		set froms [::hotmail::getFroms]
 		set fromsText ""
 		foreach {from frommail} $froms {
@@ -438,19 +676,30 @@ proc mailicon_proc {num} {
 		}
 		if { [WinDock] } {
 			winico taskbar modify $winmailicon -text $msg
-		} else {
-			configureti $mailicon -tooltip $msg
+		} elseif {[UnixDock] } {
 			bind $mailicon <Enter> [list balloon_enter %W %X %Y $msg]
 			bind $mailicon <Motion> [list balloon_motion %W %X %Y $msg]
+		} elseif {[MacDock] } {
+			::statusicon::setTooltip $mailicon $msg
 		}
 	} 
 }
 
 proc remove_icon {icon} {
-	global systemtray_exist
-	if {$systemtray_exist == 1 && $icon != 0} {
-		catch {removeti $icon}
-#                destroy $icon
+	global systemtray_exist use_tktray
+	if {$icon != 0} {
+		# Can't use UnixDock and MacDock because if we remove the icon
+		# it might be because the user disabled the 'use_tray' option
+		# so those functions would always return false...
+		if {[OnLinux] || [OnBSD]} {
+			if {$use_tktray} {
+				catch {destroy $icon}
+			} else {
+				catch {removeti $icon}
+			}
+		} elseif {[OnMac] } {
+			catch { ::statusicon::destroy $icon}
+		}
 	}
 }
 
@@ -462,188 +711,3 @@ proc restart_tray { } {
     statusicon_proc [::MSN::myStatusIs]
     
 }
-		
-
-
-
-
-
-#Begin of a clean, platform independent trayicon API that could be made a plugin, maybe a snit object
-	
-#TODO: tooltips, windows testing, possibility to query the names of all tray-icons available
-
-
-
-#######################################################################
-# loadTrayLib {}                                                      #
-#     This procedure tries to load the trayicon library, depending    #
-#     on the platform it is used on.  It returns 1 if it succeeded,   #
-#     0 on failure.  Errors are printed to stdout.                    #
-#     Depends on:                                                     #
-#       - OnWin / OnLinux / trans                                     #
-#######################################################################
-proc loadTrayLib {} {
-	
-	if { [OnWin] } {
-
-		# First try with the package winico (for the 0.6 version)
-		catch {package require Winico}
-		if {[isWinicoLoaded] } {
-			return 1
-		}
-
-		#set file name of the lib
-		set ext "[file join utils windows winico05.dll]"
-
-		#if the lib is not available, print an error on the console and return
-		if { ![file exists $ext] } {
-			status_log "[trans needwinico2]"
-			return 0
-		}
-
-		#if there is a problem loading the lib, print the error on the console and return
-		if { [catch {load $ext winico} errormsg] } {
-			status_log "$errormsg"
-			return 0
-		}
-
-	} elseif { [OnLinux] || [OnBSD] } {
-		#if there is a problem loading the lib, print the error on the console and return
-		if { [catch {package require libtray} errormsg] } {
-			status_log "[trans traynotcompiled] : $errormsg"
-			return 0
-		}
-	
-		#check if a systemtray is available, ifnot, set the "lib loaded" state off so no icons will be made
-		if { ![systemtray_exist] } {
-			status_log "[trans nosystemtray]"
-			return 0
-		}
-		
-	}
-
-	#no errors where encountered at this point, the library should be loaded and a tray available; return 1:
-	return 1	
-}
-
-#######################################################################
-# addTrayIcon {name xiconpath winiconpath {tooltip ""}                #
-#                                    {winactionhandler "nohandler"}}  #
-#     This procedure tries to add an icon to the tray.                #
-#     Vars:                                                           #
-#       - name: the internal specific name of the icon (as widget)    #
-#       - xiconpath: the path to the pixmap for X11 systems           #
-#       - winiconpath: the path to the .ico-file for windows systems  #
-#       - tooltip:  text to show when hovered by the mousepointer     #
-#       - tooltip:  handler proc for communication on windows         #
-#     Depends on:                                                     #
-#       - loadTrayLib / OnWin / OnLinux / tray lib  /                 #
-#         balloon_enter / balloon_motion / kill_balloon               #
-#######################################################################
-proc addTrayIcon {name xiconpath winiconpath {tooltip ""} {winactionhandler "nohandler"}} {
-	if {[loadTrayLib]} {
-		global defaultbackground
-
-		if { $name == "" } { set name "trayicon" }
-			
-		#Windows specific code
-		if { [OnWin] && $winiconpath != ""} {
-			set ${name} [winico create $winiconpath]
-			#add the icon
-			winico taskbar add $name -text "$tooltip" -callback "$winactionhandler %m %x %y"
-			return 1
-
-
-		#X11/Freedesktop (linux) specific code
-		} elseif { ([OnLinux] || [OnBSD]) && $xiconpath != ""} {
-			if { [winfo exists .$name] } {
-				status_log "trayicon.tcl: won't add icon $name as it already exists"
-			} else {
-				if { [loadTrayLib] } {
-					#add the icon     !! name => .name
-					set name [newti .$name -pixmap [image create photo dest_$name] -command "::trayIcon_Configure [image create photo source_$name -file $xiconpath] dest_$name"]
-
-					#TODO: balloon bindings
-					#bind .$name <Motion> [list status_log "motion"]
-					status_log $name
-					bind $name <Enter> +[list balloon_enter %W %X %Y "Test!" [::skin::loadPixmap dbusy]]
-					bind $name <Motion> +[list balloon_motion %W %X %Y "Test!" [::skin::loadPixmap dbusy]]
-					bind $name <Leave> "+set Bulle(first) 0; kill_balloon"
-				}
-			}
-			return 1
-		} else {
-			status_log "Error creating trayicon."
-			return 0
-		}
-
-	} else {
-		return 0
-	}
-}
-
-#######################################################################
-# addTrayIcon {name }                                                 #
-#     This procedure removes an existing icon from the tray.          #
-#     Vars:                                                           #
-#       - name: the internal specific name of the icon (as widget)    #
-#     Depends on:                                                     #
-#       - loadTrayLib / OnWin / OnLinux / tray lib                    #
-#######################################################################
-proc rmTrayIcon {name} {
-	if {[loadTrayLib]} {
-		#Windows specific code
-		if { [OnWin] } {
-			winico taskbar delete $name
-		#X11/Freedesktop (linux) specific code
-		} elseif { [OnLinux] || [OnBSD] } {
-			if { [catch {removeti .$name} errormsg] } {
-				status_log "$errormsg\n"
-			}
-			if { [catch {destroy .$name} errormsg] } {
-				status_log "$errormsg\n"
-			}
-		}
-	}
-}
-
-
-proc confTrayIcon {name xiconpath winiconpath {tooltip ""} {winactionhandler "nohandler"}} {
-	if {[loadTrayLib]} {
-		#Windows specific code
-		if { [OnWin] } {
-			#remove the icon
-			winico taskbar delete $name
-			set ${name} [winico create $winiconpath]
-			#readd the icon
-			winico taskbar add $name -text "$tooltip" -callback "$winactionhandler %m %x %y"			
-
-		#X11/Freedesktop (linux) specific code
-		} elseif { [OnLinux] || [OnBSD]} {
-			configureti .$name
-			image create photo source_$name -file $xiconpath
-			image create photo dest_$name
-			
-			#TODO: Change tooltip
-		}
-
-
-	}
-}
-
-
-#######################################################################
-# Aid procedures                                                      #
-#     These are no part of the API itself                             #
-#######################################################################
-#Linux only aid proc:
-proc trayIcon_Configure {imgSrc imgDst width height} {
-	$imgDst copy $imgSrc
-	if { [image width $imgSrc] > $width || [image height $imgSrc] > $height} {
-		::picture::ResizeWithRatio $imgDst $width $height
-	}
-}
-#Windows only aid proc:
-proc nohandler {win x y} {
-	status_log "Err: No icon handler given"
-}
